/**
 * Created by Bart Jolling on 7/27/13.
 */

"use strict";

var theWindow = (typeof(unsafeWindow) == "undefined") ? window : unsafeWindow;
theWindow.lou_extensions = theWindow.lou_extensions || {};

(function (lou_extensions, undefined) {

    lou_extensions.version = undefined;

    /**
     * Injects a link to a javascript file in the HEAD of a document.
     * @param {string} url - location of the javascript file to inject.
     * @param {string} mimetype - e.g. application/javascript
     * */
    lou_extensions.injectScriptLink = function (url, mimetype) {
        var scriptElement = document.createElement('script');

        if (typeof scriptElement != "undefined") {
            scriptElement.setAttribute("type", mimetype);
            scriptElement.setAttribute("src", url);
            document.getElementsByTagName("head")[0].appendChild(scriptElement);
        }
    };

    /**
     * Injects a link to a stylesheet in the HEAD of a document.
     * @param {string} url - location of the javascript file to inject.
     * @param {string} mimetype - e.g. application/javascript
     * */
    lou_extensions.injectStylesheetLink = function (url, mimetype) {
        var linkElement = document.createElement("link");

        if (typeof linkElement != "undefined") {
            linkElement.setAttribute("rel", "stylesheet");
            linkElement.setAttribute("type", mimetype);
            linkElement.setAttribute("href", url);
            document.getElementsByTagName("head")[0].appendChild(linkElement);
        }
    };

    /**
     * Injects the content of a javascript file in the HEAD of a document.
     * @param {string} jsContent - Source code of script to inject
     * @param {string} mimeType - type of script e.g. "application/javascript"
     * @param {string} scriptId - assigns an Id to the script tag
     */
    lou_extensions.injectScript = function(jsContent, mimeType, scriptId) {
        var scriptElement = document.createElement('script');

        if (typeof scriptElement != "undefined") {
            scriptElement.setAttribute("type", mimeType);
            scriptElement.setAttribute("id", scriptId);
            scriptElement.innerHTML = jsContent;
            document.getElementsByTagName("head")[0].appendChild(scriptElement);
        }
    };

    /**
     * Injects the content of a stylesheet in the HEAD of a document.
     * @param {string} cssContent - css code to inject
     * @param {string} mimeType - type of css e.g. "text/css"
     * @param {string} styleId - assigns an Id to the style tag
     */
    lou_extensions.injectStyle = function(cssContent, mimeType, styleId) {
        var styleElement = document.createElement("style");

        if (typeof styleElement != "undefined") {
            styleElement.setAttribute("type", mimeType);
            styleElement.setAttribute("id", styleId);
            styleElement.innerHTML = cssContent;
            document.getElementsByTagName("head")[0].appendChild(styleElement);
        }
    };

    /**
     * Analyse the manifest.json to find the accessible js and css and injects them.
     * @param manifest
     */
    lou_extensions.loadExtensions = function (manifest) {
        if (manifest != null && manifest.web_accessible_resources != null) {

            lou_extensions.version = manifest.version;

            var options = lou_extensions.getOptions(lou_extensions.version);
            var prefixes = lou_extensions.getEnabledPerScriptPrefix(options);

            for (var i = 0; i < manifest.web_accessible_resources.length; i++) {
                try {
                    var resource = manifest.web_accessible_resources[i];
                    var checkForPrefix = resource.split("/")[0];

                    if( prefixes[checkForPrefix] ) {
                        
                        console.log('[lou-extensions] Injecting ' + resource);

                        if (resource.indexOf('.js', resource.length - 3) !== -1) {
                            lou_extensions.injectScriptLink(chrome.extension.getURL(resource), "application/javascript");

                        } else if (resource.indexOf('.css', resource.length - 4) !== -1) {
                            lou_extensions.injectStylesheetLink(chrome.extension.getURL(resource),"text/css");
                        }
                    }

                } catch (e) {
                    console.log('[lou-extensions] - ' + e);
                }
            }
        }
    };

})(theWindow.lou_extensions);