/**
 * Created by Bart Jolling on 7/27/13.
 */
"use strict";

var theWindow = (typeof(unsafeWindow) == "undefined") ? window : unsafeWindow;
theWindow.lou_extensions = theWindow.lou_extensions || {};

(function (lou_extensions, undefined) {

    /**
     * List of available scripts and their configuration settings
     * @type {{compat: {name: string, description: string, version: string, prefix: string, home: string, enabled: boolean}, louBos: {name: string, description: string, version: string, prefix: string, home: string, enabled: boolean}, louTweak: {name: string, description: string, version: string, prefix: string, home: string, enabled: boolean}, louMerc3: {name: string, description: string, version: string, prefix: string, home: string, enabled: boolean}}}
     */
    var availableScripts = {
        compat: {
            name: "Compatibility Layer Qooxdoo 1.6",
            description: "Adds functions that disappeared (after LoU upgrade to Qooxdoo 2.1) in order to fix older unmaintained LoU scripts",
            version: "",
            prefix: "compat",
            home: "http://userscripts.org/scripts/show/138566",
            enabled: true
        },
        louBos: {
            name: "LoU BoS",
            description: "Summary, food calculator, recruitment speed calculator, combat calculator and various other features",
            version: "1.6.0",
            prefix: "louBos",
            home: "http://userscripts.org/scripts/show/84343",
            enabled: true
        },
        louTweak: {
            name: "LoU Tweak",
            description: "Keyboard shortcuts for upgrading/downgrading/building structures, MiniMap, City Layout Sharestring generator, Current / Max queue info, Incoming resources info, Overlay city layout, Extra buttons for BB-Codes",
            version: "1.7.6",
            prefix: "louTweak",
            home: "http://userscripts.org/scripts/show/80532",
            enabled: false
        },
        louMerc3: {
            name: "MERC 3",
            description: "Displays required amount of TS for boss and dungeon raiding",
            version: "3.0.2",
            prefix: "louMerc3",
            home: "https://userscripts.org/scripts/show/175597",
            enabled: false
        }
    };
    var options = null;

    lou_extensions.haveOptionsChanged = false;

    var setOptions = function (options) {
        var optionString = JSON.stringify(options);
        localStorage.setItem("lou-extensions-options", optionString);
    };

    /**
     * Tries loading the options from local storage. If they are not up-to-date for this version,
     * the function will correct and store them in local storage
     * @param expectedVersion
     */
    lou_extensions.getOptions = function (expectedVersion) {

        if (options) {
            return options;
        }

        var optionString = null;

        if (typeof (localStorage) !== "undefined") {
            optionString = localStorage.getItem("lou-extensions-options");
        }

        if (optionString) {
            var loadedOptions = JSON.parse(optionString);

            if (loadedOptions.version == expectedVersion && lou_extensions.areOptionsValid(loadedOptions)) {
                lou_extensions.haveOptionsChanged = false;
                options = loadedOptions;
            } else {
                //upgrade scenario - rebuild options, try re-using existing options
                lou_extensions.haveOptionsChanged = true;

                for (var key in loadedOptions.scripts) {
                    if (loadedOptions.scripts.hasOwnProperty(key)) {
                        var loadedScript = loadedOptions.scripts[key];
                        var availableScript = availableScripts[key];

                        if (availableScript) {
                            availableScript.enabled = loadedScript.enabled;
                        }
                    }
                }
            }
        } else {
            lou_extensions.haveOptionsChanged = true;
        }

        if (lou_extensions.haveOptionsChanged) {
            options = {
                version: expectedVersion,
                scripts: availableScripts
            };

            setOptions(options);
        }

        return options;
    };

    /**
     * Verify that the options contain the same list of scripts as in 'availableScripts'
     * @param loadedOptions
     */
    lou_extensions.areOptionsValid = function (loadedOptions) {
        var isValid = true;

        for (var key in availableScripts) {
            if (availableScripts.hasOwnProperty(key) &&
                !loadedOptions.scripts.hasOwnProperty(key)) {

                isValid = false;
                console.log("[lou-extensions] Missing script definition: " + key);
            }
        }

        return isValid;
    };

    /**
     * Returns a dictionary of prefixes of enabled scripts
     * @param currentOptions
     * @returns Returns a dictionary of prefixes of enabled scripts
     */
    lou_extensions.getEnabledPerScriptPrefix = function (currentOptions) {
        //Mandatory scripts
        var prefixes = {
            "extensions": true
        };

        //Optional scripts
        for (var key in currentOptions.scripts) {
            if (currentOptions.scripts.hasOwnProperty(key)) {
                var script = currentOptions.scripts[key];
                prefixes[script.prefix] = script.enabled;
            }
        }

        return prefixes;
    }

})(theWindow.lou_extensions);