loader.addFinishHandler(function () {

    qx.Class.define("paTweak.BossUtils", {
        type: "static",
        extend: qx.lang.Object,
        construct: function () {
            this.base(arguments);
        },
        statics: {
            BOSS_DEFENSE_STRONG: [2500, 15000, 100000, 200000, 500000, 750000, 1000000, 1500000, 2250000, 3000000],
            BOSS_DEFENSE_WEAK: [1700, 10000, 68000, 132000, 332000, 500000, 680000, 1000000, 1500000, 2000000],

            requestBossInfo: function (x, y, callback) {
                var _this = this;
                var activeCity = webfrontend.data.City.getInstance();

                var request = {
                    cityid: activeCity.getId(),
                    x: x,
                    y: y
                };

                var commandManager = webfrontend.net.CommandManager.getInstance();
                commandManager.sendCommand("GetOrderTargetInfo", request, null, function (ok, data) {
                    var info = _this.getBossInfo(data);

                    if (info) {
                        info.name = info.cn;
                        info.coords = paTweak.CombatTools.normalizeCoords(x + ":" + y);
                        callback(info);
                    }
                    else {
                        paDebug("Unable to get target info");
                    }
                });
            },
            getBossInfo: function (data) {
                // Get level
                var m = data.cn.match(/^([^:]+):(\d+)$/);
                if (m == null) {
                    return null;
                }

                var lvl = Number(m[2]);

                // Get info
                switch (data.t) {
                    case 6:
                        // Boss Forest
                        return {weakness: "c", level: lvl, water: false};
                    case 7:
                        // Boss Mountain
                        return {weakness: "i", level: lvl, water: false};
                    case 8:
                        // Boss Hill
                        return {weakness: "i", level: lvl, water: false};
                    case 12:
                        // Boss Sea
                        // Note: we need both siege and demo attacks here
                        return {weakness: "sd", level: lvl, water: true};
                    default:
                        return null;
                }
            },
            prepareAttack: function (bossInfo) {
                // Get available units
                var city = webfrontend.data.City.getInstance();
                var availUnits = paTweak.CombatTools.getAvailableUnits(city, false);

                // Which group to use?
                var units = [].concat(bossInfo.water ? availUnits.ships : availUnits.land);

                // Enrich unit list by strength
                units.forEach(function (u) {
                    var dmg = paTweak.CombatTools.getUnitDamage(u.type);
                    if (dmg > 0) {
                        u.dmg = dmg;
                    }
                });

                // Sort units by total dmg
                units.sort(function (a, b) {
                    return (b.count * b.dmg) - (a.count * a.dmg);
                });

                // Go thru units and try to issue an order
                var order;
                for (var i = 0; i < units.length; i++) {
                    order = this.getOrder(bossInfo, units[i]);
                    if (order != null) {
                        break;
                    }
                }

                // Validate
                if (order == null) {
                    throw new Error("No unit to attack with");
                }

                return order;
            },
            getOrder: function (bossInfo, unit) {
                // Boss strength
                var str;
                if (bossInfo.weakness.indexOf(unit.off) > -1) {
                    str = paTweak.BossUtils.BOSS_DEFENSE_WEAK[bossInfo.level - 1];
                } else {
                    str = paTweak.BossUtils.BOSS_DEFENSE_STRONG[bossInfo.level - 1];
                }

                // How many units we need?
                var reqCount = Math.ceil(str / unit.dmg);

                // Compare
                if (unit.count < reqCount) {
                    return null;
                }

                // Return order
                var unitOrder = qx.lang.Object.clone(unit);
                unitOrder.count = reqCount;
                return [unitOrder];
            },
            sendAttack: function (x, y, callback) {
                var _this = this;

                this.requestBossInfo(x, y, function (bossInfo) {
                    try {
                        var units = _this.prepareAttack(bossInfo);
                        paTweak.CombatTools.orderUnits(units, bossInfo.coords, 8, 1, 0, function (ok, errorCode) {
                            var error = paTweak.CombatTools.getErrorMessage(errorCode);
                            paDebug("Hunt result=" + error);

                            if (callback) {
                                callback(ok, errorCode, error);
                            }
                        });
                    } catch (e) {
                        paDebug(e);
                    }
                });
            }
        }
    });
});

