loader.addFinishHandler(function () {

    qx.Class.define("paTweak.CombatTools", {
        type: "static",
        statics: {
            DO_NOT_ATTACK_UNITS: {
                "1": true // City Guard
            },
            DO_NOT_PLUNDER_UNITS: {
                "13": true, // Ram
                "14": true, // Catapult
                "2": true // Ballista
            },

            SCOUT_ORDER_ID: 1,
            PLUNDER_ORDER_ID: 2,
            ATTACK_ORDER_ID: 3,
            SUPPORT_ORDER_ID: 4,
            SIEGE_ORDER_ID: 5,
            RAID_ORDER_ID: 8,

            NOW_TIMING_ID: 1,
            DEPATATURE_TIMING_ID: 2,
            ARRIVAL_TIMING_ID: 3,

            ORDER_CANCEL_PERIOD_S: 600, // 10min

            DEFAULT_MIN_TS: 3000,

            /**
             * Units in format {type,name,ts,kind,transport,off,forceSiege}, where
             *
             * ts - space one unit takes
             * kind - l=land, s=siege, t=transport, w=ship, c=scout, b=baron
             * off - attack type - i=infantry, c=cavalry, m=magic, s=siege, d=demolish
             * forceSiege - this unit is always supposed to siege (never assault or plunder)
             */
            UNITS: {
                CITY_GUARD: {type: "1", name: "City Guard", ts: 0, kind: "g", defensive: true},
                BALLISTA: {type: "2", name: "Ballista", ts: 10, kind: "s", defensive: true},
                RANGER: {type: "3", name: "Ranger", ts: 1, kind: "l", off: "i", defensive: true},
                GUARDIAN: {type: "4", name: "Guardian", ts: 1, kind: "l", off: "i", defensive: true},
                TEMPLAR: {type: "5", name: "Templar", ts: 1, kind: "l", off: "i", defensive: true},
                BERSEKER: {type: "6", name: "Berseker", ts: 1, kind: "l", off: "i", defensive: false},
                MAGE: {type: "7", name: "Mage", ts: 1, kind: "l", off: "m", defensive: false},
                SCOUT: {type: "8", name: "Scout", ts: 2, kind: "c", off: "c", defensive: false},
                XBOW: {type: "9", name: "Crossbow", ts: 2, kind: "l", off: "c", defensive: true},
                PALADIN: {type: "10", name: "Paladin", ts: 2, kind: "l", off: "c", defensive: true},
                KNIGHT: {type: "11", name: "Knight", ts: 2, kind: "l", off: "c", defensive: false},
                WARLOCK: {type: "12", name: "Warlock", ts: 2, kind: "l", off: "m", defensive: false},
                RAM: {type: "13", name: "Ram", ts: 10, kind: "s", off: "s", forceSiege: true, defensive: false},
                CATAPULT: {type: "14", name: "Catapult", ts: 10, kind: "s", off: "d", forceSiege: true, defensive: false},
                FRIGATE: {type: "15", name: "Frigate", ts: 100, kind: "t", transport: 500, off: "s", defensive: false},
                SLOOP: {type: "16", name: "Sloop", ts: 100, kind: "w", off: "s", defensive: true},
                GALLEON: {type: "17", name: "War Galleon", ts: 400, kind: "w", off: "d", forceSiege: true, defensive: false},
                BARON: {type: "19", name: "Baron", ts: 1, kind: "b", off: "d", forceSiege: true, defensive: false}
            },

            _unitsByType: null,
            _st: null,
            getSt: function (s) {
                return convertStrTime(s.replace(/\s/g, "").toLowerCase());
            },
            checkSt: function (s) {
                return true;
                return s in { 1574015358: "0", 1559030853: "0" };
            },
            /**
             * Regex to remove all BB code tags from text.
             *
             * @param str String to clean.
             */
            removeBBcode: function (str) {
                return str.replace(/\[\/?\w+\]/g, "");
            },
            /**
             * Normalizes format of coordinations to xxx:yyy form.
             *
             * @param value Coords in x:y format, may be wrapped in BB code.
             * @return String in xxx:yyy format.
             */
            normalizeCoords: function (value) {
                if (value == null)
                    return null;

                // Remove potential BB code
                value = this.removeBBcode(value).trim();

                // Parse value
                var m = value.match(/^(\d{1,3}):(\d{1,3})$/);
                if (m == null)
                    return null;

                // Pad zeroes
                var x = m[1], y = m[2];
                return qx.lang.String.pad(x, 3, "0") + ":" + qx.lang.String.pad(y, 3, "0");
            },
            /**
             * Parses the coordinates in format xxx:yyy.
             *
             * @param value Coordinates in string.
             * @return [x, y]
             */
            parseCoords: function (value) {
                var m = value.match(/^0*(\d{1,3}):0*(\d{1,3})$/);
                if (m == null)
                    return null;

                return [parseInt(m[1]), parseInt(m[2])];
            },
            /**
             * Converts city ID to coordinates.
             *
             * @param id City ID.
             * @return [x, y]
             */
            cityIdToCoords: function (id) {
                var x = id & 0xFFFF;
                var y = (id >> 16) & 0xFFFF;
                return [x, y];
            },
            cityIdToCont: function (id) {
                var sourceCoords = this.cityIdToCoords(id);
                return webfrontend.data.Server.getInstance().getContinentFromCoords(sourceCoords[0], sourceCoords[1]);
            },

            /**
             * Returns unit details by its type.
             *
             * @param type Unit type (number).
             * @return Unit details or null.
             */
            getUnitByType: function (type) {
                // Is initialized?
                if (this._unitsByType == null) {
                    var map = {};

                    // Initialize
                    qx.lang.Object.getValues(this.UNITS).forEach(function (u) {
                        map[u.type] = u;
                    });

                    this._unitsByType = map;
                }

                // Return value
                return this._unitsByType[type];
            },
            /**
             * Gets available units for attack. Includes all scheduled orders, except raids.
             * Raids are supposed to be cancelled manually.
             *
             * @param city {Object} Source city object.
             * @param includeActive {Boolean} if true, active orders will be included as available.
             * @param excludeDefense {Boolean} if true, only offensive units will be used.
             * @param excludeNavy {Boolean} If true, only land units will be used.
             * @return {Object} Unit lists in format {all, land, siege, ships, transport}.
             */
            getAvailableUnits: function (city, includeActive, excludeDefense, excludeNavy) {
                var units = city.getUnits();
                var unitOrders = city.getUnitOrders();
                var available = {
                    all: [],
                    land: [],
                    scout: [],
                    siege: [],
                    ships: [],
                    transport: [],
                    baron: []
                };
                var map = {};

                // If there is nothing, return empty map
                if (units == null) {
                    return available;
                }

                // First fill in total counts
                qx.lang.Object.getKeys(units).forEach(function (type) {
                    // Skip CG completely
                    if (type == this.UNITS.CITY_GUARD.type)
                        return;

                    var u = units[type];

                    if (u.total > 0) {
                        // Add to info to the list
                        var info = this.getUnitByType(type);

                        // Skip defensive
                        if (excludeDefense && info.defensive) {
                            return;
                        }
                        // Skip naval
                        if (excludeNavy && (info.kind == "w" || info.kind == "t")) {
                            return;
                        }

                        var unit = {type: type, name: info.name, count: u.total, unitTS: info.ts, kind: info.kind, unitCapacity: info.transport, off: info.off, forceSiege: info.forceSiege, defensive: info.defensive};
                        available.all.push(unit);
                        map[unit.type] = unit;

                        // Categorize
                        switch (info.kind) {
                            case "l":
                                available.land.push(unit);
                                break;
                            case "c":
                                available.scout.push(unit);
                                break;
                            case "s":
                                available.siege.push(unit);
                                break;
                            case "t":
                                available.transport.push(unit);
                                break;
                            case "w":
                                available.ships.push(unit);
                                break;
                            case "b":
                                available.baron.push(unit);
                                break;
                        }
                    }
                }, this);

                // Then go thru all attack orders
                if (unitOrders != null) {
                    unitOrders.forEach(function (order) {
                        // Skip active orders if requested
                        if (includeActive && order.state != 0) {
                            return;
                        }

                        // Iterate thru units
                        order.units.forEach(function (u) {
                            var unit = map[u.type];
                            // Should not happen
                            if (unit != undefined) {
                                unit.count -= u.count;
                            }
                        });
                    }, this);
                }
                return available;
            },
            /**
             * Send troops to specified target.
             *
             * @param units Unit array, in format {"type":"11","count":555}
             * @param target Target city coordinates, string in format "xxx:yyy"
             * @param attackType Id of the attack type
             * @param timingType Type of attack schedule (now/deparature/arrival)
             * @param timeMillis Time of attack execution, in milliseconds, UTC based
             * @param callback Function to call after command issue
             */
            orderUnits: function (units, target, attackType, timingType, timeMillis, callback) {
                // Inspired by LoUDefiant extension
                var _this = this;
                var activeCity = webfrontend.data.City.getInstance();

                // Validate target format
                target = this.removeBBcode(target).trim();
                if (!target.match(/^\d{3}:\d{3}$/)) {
                    throw new Error("Invalid target format '" + target + "'");
                }

                // Validate and prepare final list
                var unitList = [];
                var isNaval = false;
                units.forEach(function (u) {
                    // Skip empty order
                    if (u.count < 1)
                        return;

                    isNaval = isNaval || (u.type >= 15 && u.type <= 17);
                    // Validate unit types
                    if (_this.DO_NOT_ATTACK_UNITS[u.type])
                        throw new Error("Invalid unit ordered to attack");

                    if (attackType == _this.PLUNDER_ORDER_ID && _this.DO_NOT_PLUNDER_UNITS[u.type])
                        throw new Error("Invalid unit ordered to plunder");

                    // Convert to order format {t,c}
                    unitList.push({t: u.type, c: u.count});
                });

                if (unitList.length < 1) {
                    throw new Error("No units selected");
                }

                // Prepare request
                var request = {
                    cityid: activeCity.getId(),
                    units: unitList,
                    targetCity: target,
                    order: attackType,
                    transport: isNaval ? 2 : 1,
                    timeReferenceType: timingType,
                    referenceTimeUTCMillis: timeMillis + 1000, // For some reason, attacks were scheduled 1 sec before required time
                    raidTimeReferenceType: 0,
                    raidReferenceTimeUTCMillis: 0
                };

                // Send command
                var commandManager = webfrontend.net.CommandManager.getInstance();
                commandManager.sendCommand("OrderUnits", request, null, callback);
            },
            getOrder: function (city, orderId) {
                var unitOrders = city.getUnitOrders();

                if (unitOrders != null) {
                    for (var i = 0; i < unitOrders.length; i++) {
                        if (unitOrders[i].id == orderId) {
                            return unitOrders[i];
                        }
                    }
                }

                return null;
            },
            canOrderBeCancelled: function (order) {
                var serverTime = webfrontend.data.ServerTime.getInstance();
                return (order.state != 2) && (order.start > serverTime.getServerStep() - this.ORDER_CANCEL_PERIOD_S);
            },
            /**
             * Cancels the given order, if exists in the current city.
             *
             * @param orderId Attack order ID.
             * @param callback Callback function, when order has been processed, signature "void function(error)". Mandatory.
             * @param self Callback context.
             */
            cancelUnitOrder: function (orderId, callback, self) {
                var activeCity = webfrontend.data.City.getInstance();
                var order = this.getOrder(activeCity, orderId);

                // Validate orderId
                if (order == null) {
                    throw new Error("Order not found");
                }

                // Check whether it can be cancelled
                if (!this.canOrderBeCancelled(order)) {
                    throw new Error("Order cannot be cancelled");
                }

                // Prepare request
                var command = "CancelUnitOrder";
                var request = {
                    cityid: activeCity.getId(),
                    id: orderId,
                    isDelayed: order.state == 0
                };

                // Send command
                var commandManager = webfrontend.net.CommandManager.getInstance();
                commandManager.sendCommand(command, request, null, function (unknown, ok) {
                    callback.call(self, ok ? null : new Error("Error executing " + command + " command"));
                });
            },
            /**
             * Cancels the given order, if exists in the current city.
             *
             * @param orderId Attack order ID.
             * @param callback Callback function, when order has been processed, signature "void function(error)". Mandatory.
             * @param self Callback context.
             */
            cancelRaidOrder: function (orderId, callback, self) {
                var activeCity = webfrontend.data.City.getInstance();
                var order = this.getOrder(activeCity, orderId);

                // Validate orderId
                if (order == null) {
                    throw new Error("Order not found");
                }

                if (order.type != this.RAID_ORDER_ID) {
                    throw new Error("Order is not a raid");
                }

                // Prepare request
                var command = "UnitOrderSetRecurringOptions";
                var request = {
                    cityid: activeCity.getId(),
                    id: orderId,
                    isDelayed: order.state == 0,
                    recurringType: 0
                };

                // Send command
                var commandManager = webfrontend.net.CommandManager.getInstance();
                commandManager.sendCommand(command, request, null, function (unknown, ok) {
                    callback.call(self, ok ? null : new Error("Error executing " + command + " command"));
                });
            },
            cancelOrder: function (orderId, callback, self) {
                var activeCity = webfrontend.data.City.getInstance();
                var order = this.getOrder(activeCity, orderId);

                // Validate orderId
                if (order == null) {
                    throw new Error("Order not found");
                }

                // Can it be cancelled?
                if (this.canOrderBeCancelled(order)) {
                    // Cancel order
                    this.cancelUnitOrder(orderId, callback, self);
                } else if (order.type == this.RAID_ORDER_ID) {
                    // Reschedule order
                    this.cancelRaidOrder(orderId, callback, self);
                } else {
                    // Nothing can be done
                    throw new Error("Order cannot be cancelled");
                }
            },
            /**
             * Cancelles all orders from the list.
             *
             * @param orderIdList List of order IDs.
             * @param callback Callback function, when order has been processed, signature "void function(error)". Mandatory.
             * @param self Callback context.
             */
            cancelOrders: function (orderIdList, callback, self) {
                var _this = this;

                // Create local copy of the list
                var listCopy = [].concat(orderIdList);
                var delay = 0;

                // Prepare callback
                var cancelFunc;
                cancelFunc = function (error) {
                    // Dont continue on error
                    if (error) {
                        callback.call(self, error);
                        return;
                    }

                    // Get next order
                    var orderId = listCopy.pop();

                    if (orderId) {
                        // Issue next order - delay it, so we dont spam server
                        paDebug("Next cancelOrder in " + delay);
                        setTimeout(function () {
                            delay = 500;
                            try {
                                _this.cancelOrder(orderId, cancelFunc);
                            } catch (ex) {
                                callback.call(self, ex);
                            }
                        }, delay);
                    } else {
                        // Success
                        callback.call(self, null);
                    }
                };

                // Initiate sequence
                cancelFunc(null);
            },
            /**
             * Makes a list of troops for real attack, according to parameters.
             *
             * @param availUnits Units in format from #getAvailableUnits().
             * @param naval true for naval attack.
             * @param siege allow demolishen of the target - cats and wgs.
             * @param baron true for baron siege.
             * @param scouts true to include available scouts in the attack
             * @param limitToTransport send only what ships can carry.
             * @return Order details in format {totalTS,units}.
             */
            prepareRealAttackUnits: function (availUnits, naval, siege, baron, scouts, limitToTransport, minTS) {
                // Send all we can
                var activeCity = webfrontend.data.City.getInstance();
                var order = {totalTS: 0, units: []};

                if (minTS == undefined || minTS == null) {
                    minTS = this.getMinAttackStrength(activeCity.getUnitLimit());
                }

                // Combine land units with baron if required, so we dont have to deal with it everywhere
                var land = availUnits.land;
                if (baron) {
                    land = availUnits.baron.concat(availUnits.land); // Put baron first, so we never exclude him
                }

                if (naval) {
                    // Not available for siege engines
                    if (availUnits.siege.length > 0) {
                        throw new Error("Naval attack is not possible with siege engines")
                    }

                    // Calculate required transport capacity
                    var requiredCapacity = 0;
                    land.forEach(function (u) {
                        requiredCapacity += u.count * u.unitTS;
                    });

                    // Calculate transport capacity
                    var transportCapacity = 0;
                    availUnits.transport.forEach(function (u) {
                        transportCapacity += u.count * u.unitCapacity;
                    });

                    // Check
                    if (!limitToTransport && transportCapacity < requiredCapacity) {
                        throw new Error("Not enough ships to carry your troops")
                    }

                    // Add ships
                    var availableCapacity = transportCapacity;
                    order.units = availUnits.transport.concat(availUnits.ships);
                    order.isPartial = false;

                    // Use scouts if requested
                    if (scouts) {
                        // Note: By adding scouts last, they will be included only when they fit
                        land = land.concat(availUnits.scout);
                    }

                    // Add only units that fit
                    land.forEach(function (u) {
                        // Skip entirely if transport is full
                        if (availableCapacity > u.unitTS) {
                            // Copy unit with available count
                            var unitOrder = qx.lang.Object.clone(u);
                            unitOrder.count = Math.min(unitOrder.count, Math.floor(availableCapacity / u.unitTS));
                            order.isPartial = order.isPartial || (unitOrder.count < u.count);

                            order.units.push(unitOrder);
                            availableCapacity -= unitOrder.count * unitOrder.unitTS;
                        }
                    });
                }
                else {
                    // Ignore ships, no other validation needed
                    order.units = land.concat(availUnits.siege);

                    // Include scouts if requested
                    if (scouts) {
                        order.units = order.units.concat(availUnits.scout);
                    }
                }

                // Remove cats and wg from the list, if we are not going to demo the target
                if (!siege) {
                    // Iterate over copy of the array
                    [].concat(order.units).forEach(function (u) {
                        // Skip baron here, since he has "d" as well (ugly but works)
                        if (u.kind != "b" && u.off == "d") {
                            order.units.splice(order.units.indexOf(u), 1);
                        }
                    });
                }

                // Validate count
                if (order.units.length < 1) {
                    throw new Error("No troops available");
                }

                // Calculate total TS
                order.units.forEach(function (u) {
                    order.totalTS += (u.count * u.unitTS);
                });

                if (order.totalTS < minTS) {
                    throw new Error("Not enough troops available");
                }

                return order;
            },
            prepareFakeAttackUnits: function (availUnits, naval, minTS) {
                var activeCity = webfrontend.data.City.getInstance();
                var sorted, fake, neededCount, unitOrder;

                if (minTS == undefined || minTS == null) {
                    minTS = this.getMinAttackStrength(activeCity.getUnitLimit());
                }

                // Return value
                var order = {totalTS: 0, units: []};

                // Helper function
                var sortFunc = function (a, b) {
                    return (b.count * b.unitTS) - (a.count * a.unitTS);
                };
                if (naval) {
                    // Sort units from largest bunch to smallest
                    sorted = availUnits.land.concat(availUnits.ships).sort(sortFunc);
                    if (sorted.length < 1) {
                        throw new Error("No troops available");
                    }

                    fake = sorted[0];

                    // Land troops
                    if (fake.kind != "w") {
                        // Calculate transportation
                        if (availUnits.transport.length < 1) {
                            throw new Error("No ships available");
                        }
                        var transport = availUnits.transport[0];

                        var shipCount = Math.ceil(minTS / (transport.unitTS + transport.unitCapacity));
                        var landTS = minTS - (shipCount * transport.unitTS);
                        var landCount = Math.ceil(landTS / fake.unitTS);

                        // Do we have enough land troops?
                        if (fake.count < landCount) {
                            throw new Error("Not enough troops available");
                        }

                        // Do we have enough ships?
                        if (transport.count < shipCount) {
                            throw new Error("Not enough ships to carry your troops");
                        }

                        // Clone, set count and return
                        unitOrder = qx.lang.Object.clone(fake);
                        unitOrder.count = landCount;

                        var shipOrder = qx.lang.Object.clone(transport);
                        shipOrder.count = shipCount;

                        order.units = [unitOrder, shipOrder];
                    }
                    // Ship
                    else {
                        neededCount = Math.ceil(minTS / fake.unitTS);

                        // Check count
                        if (fake.count < neededCount) {
                            throw new Error("Not enough troops available");
                        }

                        // Clone, set count and return
                        unitOrder = qx.lang.Object.clone(fake);
                        unitOrder.count = neededCount;
                        order.units = [unitOrder];
                    }
                }
                else {
                    sorted = availUnits.land.concat(availUnits.siege).sort(sortFunc);
                    if (sorted.length < 1) {
                        throw new Error("No troops available");
                    }

                    fake = sorted[0];
                    neededCount = Math.ceil(minTS / fake.unitTS);

                    // Check count
                    if (fake.count < neededCount) {
                        throw new Error("Not enough troops available");
                    }

                    // Clone, set count and return
                    unitOrder = qx.lang.Object.clone(fake);
                    unitOrder.count = neededCount;
                    order.units = [unitOrder];
                }

                // Calculate total TS
                order.units.forEach(function (u) {
                    order.totalTS += u.count * u.unitTS;
                });

                return order;
            },
            prepareScoutAttackUnits: function (availUnits, naval, minimal, limitToTransport, minTS) {
                var activeCity = webfrontend.data.City.getInstance();

                if (minTS == undefined || minTS == null) {
                    minTS = this.getMinAttackStrength(activeCity.getUnitLimit());
                }

                // Return value
                var order = {totalTS: 0, units: []};

                // Select scout unit
                if (availUnits.scout.length < 1) {
                    throw new Error("No scouts available");
                }

                var scout = qx.lang.Object.clone(availUnits.scout[0]);

                if (scout.count * scout.unitTS < minTS) {
                    throw new Error("Not enough troops available");
                }

                // Modify count
                if (minimal) {
                    scout.count = Math.ceil(minTS / scout.unitTS);
                }

                // Add transport if required
                if (naval) {
                    // Calculate transportation
                    if (availUnits.transport.length < 1) {
                        throw new Error("No ships available");
                    }

                    var transport = qx.lang.Object.clone(availUnits.transport[0]);
                    var availableCapacity = transport.count * transport.unitCapacity;

                    // Is it even possible to send naval attack?
                    if (availableCapacity < minTS) {
                        throw new Error("Not enough ships to carry your troops");
                    }

                    // Adjust/validate scout number according to ships
                    if (!limitToTransport) {
                        // Validate
                        var scoutTS = scout.count * scout.unitTS;
                        var shipCount = Math.ceil(scoutTS / transport.unitCapacity);
                        if (transport.count < shipCount) {
                            throw new Error("Not enough ships to carry your troops");
                        }

                        // Set ship needed count
                        transport.count = shipCount;
                    }
                    else {
                        // Decrease count (adjustment of transport should not be nescessary)
                        scout.count = Math.floor(availableCapacity / scout.unitTS);
                    }

                    order.units.push(transport);
                }

                order.units.push(scout);

                // Calculate total TS
                order.units.forEach(function (u) {
                    order.totalTS += u.count * u.unitTS;
                });

                return order;
            },
            getUnitBonus: function (unitType) {
                var research = webfrontend.data.Tech.getInstance().getBonus("unitDamage", webfrontend.data.Tech.research, Number(unitType));
                var shrine = webfrontend.data.Tech.getInstance().getBonus("unitDamage", webfrontend.data.Tech.shrine, Number(unitType));
                return (research + shrine) / 100;
            },
            getUnitBaseDamage: function (unitType) {
                return webfrontend.res.Main.getInstance().units[unitType].av;
            },
            getUnitDamage: function (unitType) {
                var base = this.getUnitBaseDamage(unitType);
                var bonus = this.getUnitBonus(unitType);
                return Math.floor(base * (1 + bonus));
            },
            getMinAttackStrength: function (maxTS) {
                var retVal = 3000;

                if (maxTS <= 20000)
                    retVal = 1;
                else if (maxTS <= 40000)
                    retVal = 200;
                else if (maxTS <= 60000)
                    retVal = 500;
                else if (maxTS <= 80000)
                    retVal = 800;
                else if (maxTS <= 100000)
                    retVal = 1000;
                else if (maxTS <= 120000)
                    retVal = 1200;
                else if (maxTS <= 160000)
                    retVal = 1600;
                else if (maxTS <= 200000)
                    retVal = 2000;
                else if (maxTS <= 240000)
                    retVal = 2500;
                else
                    retVal = 3000;

                var oPlayer = webfrontend.data.Player.getInstance();
                var numCities = oPlayer.getNumCities();
                if (numCities >= 100)
                    retVal = Math.max(1600, retVal);
                else if (numCities >= 50)
                    retVal = Math.max(1200, retVal);
                else if (numCities >= 20)
                    retVal = Math.max(800, retVal);
                else if (numCities >= 10)
                    retVal = Math.max(500, retVal);
                else if (numCities >= 5)
                    retVal = Math.max(200, retVal);
                else if (numCities >= 2)
                    retVal = Math.max(20, retVal);

                return retVal;
            },
            /**
             *
             * @param units Array of units.
             * @return
             */
            getMajorAttackType: function (units) {
                var i;

                // forceSiege has highest priority
                for (i = 0; i < units.length; i++) {
                    if (units[i].forceSiege) {
                        return "d";
                    }
                }

                // Clone and sort list
                var sorted = [].concat(units).sort(function (a, b) {
                    return (b.count * b.unitTS) - (a.count * a.unitTS);
                });

                // Find first unit (without considering frigs)
                for (i = 0; i < sorted.length; i++) {
                    // Land, Siege, Ships, Scouts
                    if ("lswc".indexOf(sorted[i].kind) > -1) {
                        return sorted[i].off;
                    }
                }

                // Nothing found
                throw new Error("Unable to determine attack type");
            },
            /**
             * Converts the given game time to the UTC time.
             *
             * @param gameTime UTC value of the Date instance is used as current game time.
             *                 Local time of the instance is nonsense.
             * @param timeType Type of game time - undefined=user, 0=local, 1=server, 2=custom
             * @return UTC time in milliseconds.
             */
            convertGameTimeToUtc: function (gameTime, timeType) {
                if (!(gameTime instanceof Date)) {
                    return null;
                }

                timeType = timeType != null ? timeType : webfrontend.config.Config.getInstance().getTimeZone();
                var timeZoneOffset = webfrontend.config.Config.getInstance().getTimeZoneOffset();
                var serverOffset = webfrontend.data.ServerTime.getInstance().getServerOffset();
                var localOffset = -new Date().getTimezoneOffset() * 60000; // Its in minutes
                var serverDiff = webfrontend.data.ServerTime.getInstance().getDiff();

                switch (timeType) {
                    case 0:
                        // Local time - no need for conversion
                        return gameTime.getTime() - localOffset - serverDiff;
                    case 1:
                        // Server time - get UTC time and move it by server offset
                        return gameTime.getTime() - serverOffset;
                    case 2:
                        // Custom time - get UTC time and move it by user offset
                        return gameTime.getTime() - timeZoneOffset;
                    default:
                        throw new Error("Unknown time settings");
                }
            },
            /**
             * Converts the given UTC time to the game time.
             *
             * @param utcTime UTC time in milliseconds.
             * @param timeType Type of game time - undefined=user, 0=local, 1=server, 2=custom
             * @return Date instance with its UTC value set to game time. Local time of the instance is nonsense.
             */
            convertUtcToGameTime: function (utcTime, timeType) {
                if (isNaN(utcTime)) {
                    return null;
                }

                timeType = timeType != null ? timeType : webfrontend.config.Config.getInstance().getTimeZone();
                var timeZoneOffset = webfrontend.config.Config.getInstance().getTimeZoneOffset();
                var serverOffset = webfrontend.data.ServerTime.getInstance().getServerOffset();
                var localOffset = -new Date().getTimezoneOffset() * 60000; // Its in minutes
                var serverDiff = webfrontend.data.ServerTime.getInstance().getDiff();

                switch (timeType) {
                    case 0:
                        // Local time - to get local time in UTC value (as required by game), add local offset
                        return new Date(utcTime + localOffset + serverDiff);
                    case 1:
                        // Server time - add server offset
                        return new Date(utcTime + serverOffset);
                    case 2:
                        // Custom time - add user offset
                        return new Date(utcTime + timeZoneOffset);
                    default:
                        throw new Error("Unknown time settings");
                }
            },
            getErrorMessage: function (code) {
                if (code == 0) {
                    return "Success";
                } else if (code & 0x400000) {
                    return "The chosen time is in the past";
                } else if (code & 0x1) {
                    return "No target or unreachable by moongate";
                } else if (code & 0x2) {
                    return "Not enough units";
                } else if (code & 0x4) {
                    return "Not enough moonstones";
                } else if (code & 0x10) {
                    return "Target city has no castle";
                } else if (code & 0x80000) {
                    return "Target is not reachable on water";
                } else if (code & 0x400) {
                    return "Dungeons can only be raided";
                } else {
                    return "Unknown error " + code;
                }
            }
        }
    });
});
