loader.addFinishHandler(function () {

    qx.Class.define("paTweak.pakMap", {
        type: "static",

        statics: {
            app: null,
            res: null,
            tr: {
                "en": { "weak": "Weakness" },
                "de": { "weak": "Schwäche" },
                "pl": { "weak": "слабость" }
            },
            keyName: "Name:",
            keyType: "Type:",
            keyLevel: "Level:",
            keyProgress: "Progress:",

            htmlHeader: '<table cellspacing="0"><tr><td width="75">',
            htmlNewCell: "</td><td>",

            lblWeakness: 0,
            lblProgressTS: 0,
            lblUnitTS: 0,

            keyDragon: "",
            keyHydra: "",
            keyMoloch: "",
            keyOctopus: "",
            keyForestDungeon: "",
            keyMountainDungeon: "",
            keyHillDungeon: "",
            keySeaEncounter: "",

            idxBerserkers: 6,
            idxMages: 7,
            idxPaladins: 10,
            idxKnights: 11,
            idxWarlocks: 12,
            idxSloops: 16,
            idxWarGalleons: 17,

            listUnitIndexes: [6, 7, 10, 11, 12, 16, 17],
            listUnitNames: [],
            listUnitTSConversions: [],
            listAttackTypes: [],

            lookupWeaknesses: null,

            init: function (app, res, language) {

                //Accept Input
                paTweak.pakMap.app = app;
                paTweak.pakMap.res = res;

                if (language != "de" && language != "pl") {
                    language = "en";
                }

                //Get translated keys to identify information in existing tooltip
                paTweak.pakMap.keyName = app.tr("tnf:name:").toString();
                paTweak.pakMap.keyType = app.tr("tnf:type:").toString();
                paTweak.pakMap.keyLevel = app.tr("tnf:level:").toString();
                paTweak.pakMap.keyProgress = app.tr("tnf:progress:").toString();

                //Build labels for display in tooltip
                paTweak.pakMap.lblWeakness = paTweak.pakMap.tr[language]["weak"] + ':' + paTweak.pakMap.htmlNewCell;
                paTweak.pakMap.lblProgressTS = "TS + pct:" + paTweak.pakMap.htmlNewCell;
                paTweak.pakMap.lblUnitTS = "Unit TS:" + paTweak.pakMap.htmlNewCell;

                //Unit Types
                for (var idx = 0; idx < paTweak.pakMap.listUnitIndexes.length; idx++) {
                    var unitIdx = paTweak.pakMap.listUnitIndexes[idx];
                    paTweak.pakMap.listUnitNames[unitIdx] = res.units[unitIdx].dn + ":" + paTweak.pakMap.htmlNewCell;
                }

                //Attack Types
                var lat = [];
                lat[paTweak.pakMap.idxBerserkers] = 1; // infantry
                lat[paTweak.pakMap.idxMages] = 4;      // magic
                lat[paTweak.pakMap.idxPaladins] = 2;   // cavalry
                lat[paTweak.pakMap.idxKnights] = 2;    // cavalry
                lat[paTweak.pakMap.idxWarlocks] = 4;   // magic
                lat[paTweak.pakMap.idxSloops] = 3;     // artillery
                lat[paTweak.pakMap.idxWarGalleons] = 3;// artillery

                paTweak.pakMap.listAttackTypes = lat;

                //Dungeon Types
                paTweak.pakMap.keyDragon = res.dungeons["6"].dn;
                paTweak.pakMap.keyHydra = res.dungeons["8"].dn;
                paTweak.pakMap.keyMoloch = res.dungeons["7"].dn;
                paTweak.pakMap.keyOctopus = res.dungeons["12"].dn;
                paTweak.pakMap.keyForestDungeon = res.dungeons["5"].dn;
                paTweak.pakMap.keyMountainDungeon = res.dungeons["4"].dn;
                paTweak.pakMap.keyHillDungeon = res.dungeons["3"].dn;
                paTweak.pakMap.keySeaEncounter = res.dungeons["2"].dn;

                //Weaknesses
                var luw = {};
                luw[paTweak.pakMap.keySeaEncounter] = 3;    // artillery
                luw[paTweak.pakMap.keyHillDungeon] = 4;     // magic
                luw[paTweak.pakMap.keyMountainDungeon] = 1; // infantry
                luw[paTweak.pakMap.keyForestDungeon] = 2;   // cavalry;
                luw[paTweak.pakMap.keyDragon] = 2;          // cavalry;
                luw[paTweak.pakMap.keyMoloch] = 4;          // magic
                luw[paTweak.pakMap.keyHydra] = 1;           // infantry
                luw[paTweak.pakMap.keyOctopus] = 3;         // artillery

                paTweak.pakMap.lookupWeaknesses = luw;

                //Unit TS conversion
                var utsc = [];
                utsc[paTweak.pakMap.idxBerserkers] = 1;
                utsc[paTweak.pakMap.idxMages] = 0.72;
                utsc[paTweak.pakMap.idxPaladins] = 0.83;
                utsc[paTweak.pakMap.idxKnights] = 0.55;
                utsc[paTweak.pakMap.idxWarlocks] = 0.42;
                utsc[paTweak.pakMap.idxSloops] = 0.03;
                utsc[paTweak.pakMap.idxWarGalleons] = 0.003;

                paTweak.pakMap.listUnitTSConversions = utsc;
            },

            parseToolTipText: function(tooltipText) {
                var parsedToolTip = {
                    title : "",
                    rowCount : 0
                };

                var tooltipDoc = qx.xml.Document.fromString(tooltipText);
                var tooltipRows = tooltipDoc.getElementsByTagName("tr");

                //Add additional info
                parsedToolTip.rowCount = tooltipRows.length;

                if(tooltipRows.length == 0)
                    return parsedToolTip;

                //Transform html table to json
                for(var row in tooltipRows) {
                    if( tooltipRows[row].childNodes ){
                        var key = tooltipRows[row].childNodes[0].textContent;
                        parsedToolTip[key] = tooltipRows[row].childNodes[1].textContent;
                    }
                }

                //determine title - assuming in row[0]
                parsedToolTip.title = tooltipRows[0].childNodes[0].textContent;

                return parsedToolTip;
            },

            isBossTooltip: function (type) {
                return(type == paTweak.pakMap.keyName )
            },

            isDungeonTooltip: function (type) {
                return(type == paTweak.pakMap.keyType )
            },

            toolTipAppear: function () {
                try {
                    var toolTip = paTweak.pakMap.app.worldViewToolTip;
                    var mode = toolTip.getMode();
                    var tooltipText = toolTip.getLabel();

                    if (mode == 'c' || mode == 'd' || tooltipText == null) {
                        return;
                    }

                    var parsedToolTip = paTweak.pakMap.parseToolTipText(tooltipText);

                    // Type of tooltip: Boss, Dungeon, Coordinates, ...
                    var title = parsedToolTip.title;
                    if ( !paTweak.pakMap.isBossTooltip(title) && !paTweak.pakMap.isDungeonTooltip(title) ) {
                        return;
                    }

                    var dungeonType = parsedToolTip[title];
                    var dungeonLevel = parsedToolTip[paTweak.pakMap.keyLevel];

                    if( typeof(dungeonType) == "undefined" || typeof(dungeonLevel) == "undefined") {
                        return;
                    }

                    var level = parseInt(dungeonLevel);
                    var weaknessIdx = paTweak.pakMap.lookupWeaknesses[dungeonType];
                    var weakness = paTweak.pakMap.res.attackTypes[weaknessIdx].dn;

                    var sb = null;

                    if (paTweak.pakMap.isBossTooltip(title)) { // Name:
                        const bossKill = [50, 300, 2000, 4000, 10000, 15000, 20000, 30000, 45000, 60000];
                        var startIdx;
                        var stopIdx;

                        sb = new qx.util.StringBuilder(150);

                        sb.add(tooltipText, paTweak.pakMap.htmlHeader, paTweak.pakMap.lblWeakness, weakness, "</td></tr>");

                        if (dungeonType == paTweak.pakMap.keyOctopus) {
                            startIdx = 5;
                            stopIdx = paTweak.pakMap.listUnitIndexes.length;
                        } else {
                            startIdx = 0;
                            stopIdx = 5;
                        }

                        for (var idx = startIdx; idx < stopIdx; idx++) {
                            var unitIdx = paTweak.pakMap.listUnitIndexes[idx];

                            var research = webfrontend.data.Tech.getInstance().getBonus("unitDamage",
                                webfrontend.data.Tech.research, unitIdx);

                            var shrine = webfrontend.data.Tech.getInstance().getBonus("unitDamage",
                                webfrontend.data.Tech.shrine, unitIdx);

                            var bonus = 1 + (research + shrine) / 100;

                            var count = (bossKill[ level - 1] / bonus)
                                * paTweak.pakMap.listUnitTSConversions[unitIdx];

                            if (weaknessIdx == paTweak.pakMap.listAttackTypes[unitIdx]) {
                                count *= 0.67;
                            }

                            count = webfrontend.gui.Util.formatNumbers(parseInt(count));

                            sb.add("<tr><td>", paTweak.pakMap.res.units[unitIdx].dn, ":</td><td>", count, "</td></tr>")
                        }

                        sb.add("</table>");

                        toolTip.setLabel(sb.get());

                    } else if (paTweak.pakMap.isDungeonTooltip(title)) { // Type:

                        const dungeonKill = [10, 100, 450, 1500, 3500, 6000, 13000, 20000, 35000, 60000];

                        // Progress
                        var progressValue = parsedToolTip[paTweak.pakMap.keyProgress];
                        var progress = 100;

                        if( typeof(progressValue) != "undefined") {
                            //remove the percentage
                            progressValue = progressValue.slice(0, -1);
                            progress = webfrontend.gui.Util.formatNumbers(
                                parseInt((progressValue * 0.0175 + 1.0875) * dungeonKill[ level - 1])
                            );
                        }

                        var unitTS = webfrontend.gui.Util.formatNumbers(dungeonKill[ parseInt(level) - 1]);

                        sb = new qx.util.StringBuilder(150);

                        sb.add(tooltipText, paTweak.pakMap.htmlHeader, paTweak.pakMap.lblWeakness, weakness, "</td></tr>",
                            "<tr><td>", paTweak.pakMap.lblUnitTS, unitTS, "</td></tr>",
                            "<tr><td>", paTweak.pakMap.lblProgressTS, progress, "</td></tr>",
                            "</table>");

                        toolTip.setLabel(sb.get());
                    }

                } catch (e) {
                    console.error(e);
                }
            }
        }
    });
});