loader.addFinishHandler(function () {

    qx.Class.define("paTweak.ui.CancelOrderPanel", {
        extend: qx.ui.container.Composite,
        construct: function () {
            this.base(arguments);
            this.buildUI();
        },
        statics: {
            /**
             * Returns list of order IDs, filtered by provided function.
             *
             * @param filterFunc Filter function, with signature "boolean function(order)". Return true to include the order.
             * @return Array of order IDs.
             */
            getOrderList: function (filterFunc) {
                var activeCity = webfrontend.data.City.getInstance();
                var unitOrders = activeCity.getUnitOrders();

                var idList = [];

                if (unitOrders != null) {
                    unitOrders.forEach(function (order) {
                        if (filterFunc(order)) {
                            idList.push(order.id);
                        }
                    });
                }

                return idList;
            },
            cancelAll: function (callback, self) {
                // Get list
                var orderList = this.getOrderList(function (order) {
                    return paTweak.CombatTools.canOrderBeCancelled(order)
                        || (order.type == paTweak.CombatTools.RAID_ORDER_ID && order.recurringType != 0);
                });
                // Issue order
                paDebug("Orders to cancel: " + orderList.length);
                paTweak.CombatTools.cancelOrders(orderList, callback, self);
            },
            cancelAllRaids: function (callback, self) {
                // Get list
                var orderList = this.getOrderList(function (order) {
                    return order.type == paTweak.CombatTools.RAID_ORDER_ID
                        && (order.recurringType != 0 || paTweak.CombatTools.canOrderBeCancelled(order));
                });
                // Issue order
                paDebug("Orders to cancel: " + orderList.length);
                paTweak.CombatTools.cancelOrders(orderList, callback, self);
            }
        },
        members: {
            _cancelAllButton: null,
            _cancelRaidsButton: null,

            buildUI: function () {
                this.setLayout(new qx.ui.layout.VBox(5));

                var firstRow = new qx.ui.container.Composite();
                firstRow.setLayout(new qx.ui.layout.HBox());
                firstRow.set({width: 75});

                // Return By
                this._returnByButton = new qx.ui.form.Button("Rtn");
                this._returnByButton.set({width: 30, maxWidth: 30, appearance: "button-text-small", toolTipText: "All raids return by XX:XX:XX"});
                this._returnByButton.addListener("execute", this.returnBy, this);

                // Cancel All
                this._cancelAllButton = new qx.ui.form.Button("C All");
                this._cancelAllButton.set({width: 45, maxWidth: 45, appearance: "button-text-small", toolTipText: "Cancel all orders. Careful!"});
                this._cancelAllButton.addListener("execute", this.cancelAll, this);

                // Cancel Raids
                this._cancelRaidsButton = new qx.ui.form.Button("Cancel Raids");
                this._cancelRaidsButton.set({width: 75, maxWidth: 75, appearance: "button-text-small", toolTipText: "Cancel all raid orders."});
                this._cancelRaidsButton.addListener("execute", this.cancelAllRaids, this);

                // Add to layout
                this.add(this._cancelRaidsButton);
                firstRow.add(this._returnByButton);
                firstRow.add(this._cancelAllButton);
                this.add(firstRow);
            },
            _setButtonsEnabled: function (value) {
                this._cancelAllButton.setEnabled(value);
                this._cancelRaidsButton.setEnabled(value);
                this._returnByButton.setEnabled(value);
            },
            returnBy: function () {
                var dialog = paTweak.ui.ReturnByWindow.getInstance();
                dialog.center();
                dialog.show();
            },
            cancelAll: function () {
                // Check user
                if (!confirm("Do you want to cancel all orders?")) {
                    return;
                }

                this._setButtonsEnabled(false);

                this.self(arguments).cancelAll(function (error) {
                    this._setButtonsEnabled(true);

                    if (error) {
                        paDebug(error);
                    }
                }, this);
            },
            cancelAllRaids: function () {
                this._setButtonsEnabled(false);

                this.self(arguments).cancelAllRaids(function (error) {
                    this._setButtonsEnabled(true);

                    if (error) {
                        paDebug(error);
                    }
                }, this);
            }
        }
    });
});
