loader.addFinishHandler(function () {

    qx.Class.define("paTweak.ui.CombatWindow", {
        type: "singleton",
        extend: qx.ui.window.Window,
        construct: function () {
            this.base(arguments, "Combat Tool");

            // Build UI
            this._rows = [];
            this.buildUI();

            // Load prev config
            this.loadData();

            // Listeners
            this._listener_cityChanged = webfrontend.data.City.getInstance().addListener("changeVersion", function () {
                if (!this.isVisible()) {
                    return;
                }

                this.refresh();
                this._setActionEnabled(true);
                this._lock_safeguard = null;
            }, this);

            this.addListener("appear", function () {
                this.refresh();
                this.resetMessage();
            }, this);

            this.addListener("changeActive", function (e) {
                if (!e.getData()) {
                    this.storeData();
                }
            }, this);
        },
        destruct: function () {
            var city = webfrontend.data.City.getInstance();
            if (this._listener_cityChanged) city.removeListenerById(this._listener_cityChanged);
        },
        members: {
            _addButton: null,
            _resetButton: null,
            _messageLabel: null,
            _availableLabel: null,
            _includeActive: null,
            _allowPartial: null,
            _useScouts: null,
            _useSmallestForFakes: null,
            _excludeDefenseCheck: null,
            _forceMsCheck: null,
            _travelModeGroup: null,

            _rows: null,

            _magicTime: null,
            _infTime: null,
            _cavTime: null,
            _siegeTime: null,
            _copyButton: null,

            _listener_cityChanged: null,
            _lock_safeguard: null,

            buildUI: function () {
                this.setLayout(new qx.ui.layout.VBox(5));
                this.set({allowMaximize: false, allowMinimize: false, showMaximize: false, showMinimize: false,
                    showStatusbar: false, showClose: false, contentPadding: 5, useMoveFrame: true, resizable: false});

                webfrontend.gui.Util.formatWinClose(this);

                // Message
                this._messageLabel = new qx.ui.basic.Label();
                this._messageLabel.set({textColor: "#D10600", wrap: true});
                this.add(this._messageLabel);

                // Times
                this._magicTime = new paTweak.ui.components.TimePicker("Magic");
                this._cavTime = new paTweak.ui.components.TimePicker("Cavalry");
                this._infTime = new paTweak.ui.components.TimePicker("Infantry");
                this._siegeTime = new paTweak.ui.components.TimePicker("Siege");

                this._copyButton = new qx.ui.form.Button("Copy");
                this._copyButton.set({appearance: "button-text-small"});
                this._copyButton.addListener("execute", this.copyTimes, this);

                var firstTimeRow = new qx.ui.container.Composite();
                firstTimeRow.setLayout(new qx.ui.layout.HBox(5));
                firstTimeRow.add(this._magicTime);
                firstTimeRow.add(this._copyButton);

                // Put it in standalone box
                var timesBox = new qx.ui.container.Composite(new qx.ui.layout.VBox(5));
                timesBox.add(firstTimeRow);
                timesBox.add(this._cavTime);
                timesBox.add(this._infTime);
                timesBox.add(this._siegeTime);

                // Import/Export button
                var importButton = new qx.ui.form.Button("Import/Export");
                importButton.set({appearance: "button-text-small", allowGrowX: false, toolTipText: "Import or export attacks configuration."});
                importButton.addListener("execute", function () {
                    var win = paTweak.ui.CombatWindowExport.getInstance();
                    win.center();
                    win.open();
                }, this);

                // Reset button
                this._resetButton = new qx.ui.form.Button("Reset");
                this._resetButton.set({appearance: "button-text-small", allowGrowX: false, toolTipText: "Resets all values in the dialog."});
                this._resetButton.addListener("execute", function () {
                    if (confirm("Are you sure you want to throw away all your plans?")) {
                        this.reset();
                    }
                }, this);

                var buttonsRow = new qx.ui.container.Composite(new qx.ui.layout.HBox(5));
                buttonsRow.add(importButton);
                buttonsRow.add(this._resetButton);

                // Include check
                this._includeActive = new qx.ui.form.CheckBox("Include units out of the city");
                this._includeActive.setToolTipText("If checked, units currently out of the city (raiding/plundering etc) will be included into commands. You are supposed to get them home in time by yourself.");
                this._includeActive.initValue(true);
                this._includeActive.addListener("changeValue", this.refresh, this);

                // Allow partial check
                this._allowPartial = new qx.ui.form.CheckBox("Allow partial naval attack");
                this._allowPartial.setToolTipText("When there are not enough Frigates to carry your troops, it allows to send only part of the army that will fit on the ships. Has no effect on land attacks.");

                // Use scouts
                this._useScouts = new qx.ui.form.CheckBox("Include scouts in the attacks");
                this._useScouts.setToolTipText("All available scouts will be sent along other units. If there will be enough scouts, they will be also used for fakes.");
                this._useScouts.setValue(true);
                this._useScouts.addListener("changeValue", this.refresh, this);

                // Use smallest for fakes
                this._useSmallestForFakes = new qx.ui.form.CheckBox("Prefer smallest stack for fakes instead of largest");
                this._useSmallestForFakes.setToolTipText("By default, unit you have the most of is used for fakes. This changes the order.");
                this._useSmallestForFakes.setEnabled(false);
                this._useSmallestForFakes.exclude(); // Hidden for now

                // Force 3000 min score
                this._forceMsCheck = new qx.ui.form.CheckBox("Always use 3000 min TS");
                this._forceMsCheck.set({toolTipText: "Always use 3000 min TS."});
                this._forceMsCheck.setValue(true);

                // Exclude Defense
                this._excludeDefenseCheck = new qx.ui.form.CheckBox("Exclude Defense");
                this._excludeDefenseCheck.set({toolTipText: "Don't use defensive troops."});
                this._excludeDefenseCheck.setValue(true);
                this._excludeDefenseCheck.addListener("changeValue", this.refresh, this);

                // Travel mode
                var travelModeLabel = new qx.ui.basic.Label("Travel mode");
                var autoMode = new qx.ui.form.RadioButton("Auto");
                autoMode.set({model: "auto", toolTipText: "Units will be sent on foot if the target is on the same continent. Otherwise ships will be used."});
                var navyMode = new qx.ui.form.RadioButton("Navy");
                navyMode.set({model: "navy", toolTipText: "Units will be send on Frigates even to the target on the same continent. Does not affect ships."});
                var landMode = new qx.ui.form.RadioButton("Land (Moongate)");
                landMode.set({model: "land", toolTipText: "Units will be sent on foot even if the target is on different continent. Does not use ships at all."});

                var travelModeContainer = new qx.ui.container.Composite(new qx.ui.layout.Grid(5, 2));
                travelModeContainer.add(travelModeLabel, {row: 0, column: 0});
                travelModeContainer.add(autoMode, {row: 0, column: 1});
                travelModeContainer.add(navyMode, {row: 1, column: 1});
                travelModeContainer.add(landMode, {row: 2, column: 1});

                this._travelModeGroup = new qx.ui.form.RadioGroup(autoMode, navyMode, landMode);
                this._travelModeGroup.addListener("changeSelection", this.refresh, this);

                // Layout
                var optionsBox = new qx.ui.container.Composite(new qx.ui.layout.VBox(5));
                optionsBox.add(buttonsRow);
                optionsBox.add(this._includeActive);
                optionsBox.add(this._allowPartial);
                optionsBox.add(this._useScouts);
                optionsBox.add(this._useSmallestForFakes);
                optionsBox.add(this._forceMsCheck);
                optionsBox.add(this._excludeDefenseCheck);
                optionsBox.add(travelModeContainer);

                var outerBox = new qx.ui.container.Composite(new qx.ui.layout.HBox(40));
                outerBox.add(timesBox);
                outerBox.add(optionsBox);

                this.add(outerBox);

                // Units
                var availableLabel = this._availableLabel = new qx.ui.basic.Label();
                availableLabel.set({width: 250, wrap: true});

                var refreshButton = new qx.ui.form.Button("Refresh");
                refreshButton.set({appearance: "button-text-small"});
                refreshButton.addListener("execute", this.refresh, this);

                var resetCounterButton = new qx.ui.form.Button("RC");
                resetCounterButton.set({appearance: "button-text-small", toolTipText: "Reset the indicative counter."});
                resetCounterButton.addListener("execute", this.resetCounter, this);

                var availControl = new qx.ui.container.Composite();
                availControl.setLayout(new qx.ui.layout.HBox(5));
                availControl.add(availableLabel);
                availControl.add(refreshButton);
                availControl.add(new qx.ui.core.Widget().set({height: 1}), {flex: 1});
                availControl.add(resetCounterButton);
                this.add(availControl);

                // Add button
                var addButton = this._addButton = new qx.ui.form.Button("Add");
                addButton.set({appearance: "button-text-small", allowGrowX: false, toolTipText: "Adds new target field."});
                addButton.addListener("execute", this.addRow, this);

                this.add(addButton);

                // Note
                var noteLabel = new qx.ui.basic.Label("<em>Note: Send fake before real attacks.</em>");
                noteLabel.setRich(true);
                this.add(noteLabel);

                // First data row
                this.addRow();
            },
            addRow: function () {
                var row = new paTweak.ui.components.AttackOrder();
                row.addListener("attack", this.onAttack, this);

                this.addBefore(row, this._addButton);
                this._rows.push(row);

                if (this._rows.length > 15) {
                    this._addButton.setEnabled(false);
                }

                return row;
            },
            _removeRow: function (row) {
                // Remove from window
                this.remove(row);
                // Dispose it
                row.dispose();
            },
            _setActionEnabled: function (value) {
                this._rows.forEach(function (row) {
                    row.setActionEnabled(value);
                });
            },
            reset: function () {
                // Delete rows
                this._rows.forEach(this._removeRow, this);
                this._rows = [];

                // We need at least one row
                this.addRow();
                this._addButton.setEnabled(true);

                // Reset times
                this._magicTime.resetValue();
                this._cavTime.resetValue();
                this._infTime.resetValue();
                this._siegeTime.resetValue();

                // Options
                this._includeActive.setValue(false);
                this._allowPartial.setValue(false);
                this._useScouts.setValue(true);
                this._useSmallestForFakes.setValue(false);
                this._forceMsCheck.setValue(true);
                this._excludeDefenseCheck.setValue(true);
                this._travelModeGroup.resetSelection();
            },
            resetCounter: function () {
                this._rows.forEach(function (row) {
                    row.resetCount();
                }, this);
            },
            refresh: function () {
                try {
                    var city = webfrontend.data.City.getInstance();

                    // Parameters
                    var combatTools = paTweak.CombatTools;
                    var includeActive = this._includeActive.getValue();
                    var excludeDefense = this._excludeDefenseCheck.getValue();
                    var travelMode = this.getTravelMode();
                    var excludeNavy = (travelMode == "land");

                    // Get units
                    var availUnits = combatTools.getAvailableUnits(city, includeActive, excludeDefense, excludeNavy);

                    // Format it
                    var text = "";

                    availUnits.all.forEach(function (u) {
                        if (u.count > 0) {
                            if (text.length > 0)
                                text += ", ";
                            text += u.count + " " + u.name;
                        }
                    });

                    if (text.length == 0) {
                        text = "No troops available";
                    }

                    this._availableLabel.setValue(text);
                }
                catch (ex) {
                    paDebug(ex);
                    this.setMessage(ex);
                }
            },
            onAttack: function (e) {
                var _this = this;

                try {
                    // Assemble attack info
                    var data = e.getData();
                    var target = e.getTarget();
                    var attack = this.getAttackDetails(data.target, data.type, data.attack);

                    // Validate TS
                    var minTS = this.getMinAttackTS();
                    if (minTS > 0 && data.attack != "fake" && attack.attackTS < minTS) {
                        //noinspection ExceptionCaughtLocallyJS
                        throw new Error("Minimal troop count for the attack not met");
                    }

                    // Disable buttons - they will be enabled automatically on city update
                    _this._setActionEnabled(false);
                    var lockId = _this._lock_safeguard = Math.random();
                    paDebug("Attack lock = " + lockId);
                    setTimeout(function () {
                        paDebug("Attack lock timeout: _this._lock_safeguard=" + _this._lock_safeguard + " lockId=" + lockId);
                        if (_this._lock_safeguard == lockId) {
                            _this._setActionEnabled(true);
                        }
                    }, 10000);

                    // Send attack order
                    paTweak.CombatTools.orderUnits(attack.units, attack.target, attack.type, attack.timingType, attack.time, function (ok, errorCode) {
                        if (errorCode.r0 == 0) {
                            // Nice message
                            var msg = attack.isPartial ? "Partial attack sent" : "Attack sent";
                            msg += " (" + attack.attackTS + " TS)";
                            _this.setMessage(msg);

                            // Simple counter
                            target.addCount((attack.attackType || "").toUpperCase());
                        }
                        else {
                            paDebug(errorCode.r0 + ":" + errorCode.r1);
                            var error = paTweak.CombatTools.getErrorMessage(errorCode.r0);
                            _this.setMessage("Unable to dispatch troops: " + error);
                            _this._setActionEnabled(true);
                            _this._lock_safeguard = null;
                        }
                    });
                }
                catch (ex) {
                    this.setMessage(ex);
                }

                // Store data
                this.storeData();
            },
            getTravelMode: function () {
                var sel = this._travelModeGroup.getSelection()[0];
                return sel ? sel.getModel() : null;
            },
            getAttackTimes: function () {
                var combatTools = paTweak.CombatTools;
                var siege = combatTools.convertGameTimeToUtc(this._siegeTime.getValue());

                return {
                    i: combatTools.convertGameTimeToUtc(this._infTime.getValue()),
                    m: combatTools.convertGameTimeToUtc(this._magicTime.getValue()),
                    c: combatTools.convertGameTimeToUtc(this._cavTime.getValue()),
                    s: siege,
                    d: siege
                };
            },
            getAttackDetails: function (target, type, attack) {
                var city = webfrontend.data.City.getInstance();
                var server = webfrontend.data.Server.getInstance();
                var combatTools = paTweak.CombatTools;

                // Parameters
                var includeActive = this._includeActive.getValue();
                var useScouts = this._useScouts.getValue();
                var excludeDefense = this._excludeDefenseCheck.getValue();
                var allowPartial = this._allowPartial.getValue();
                var travelMode = this.getTravelMode();
                var minTS = this._forceMsCheck.getValue() ? paTweak.CombatTools.DEFAULT_MIN_TS : null;

                // Get available units
                var availUnits = combatTools.getAvailableUnits(city, includeActive, excludeDefense, travelMode == "land");

                // Determine, whether we need naval attack
                var naval = (travelMode == "navy" || availUnits.ships.length > 0);
                if (!naval && travelMode != "land") {
                    var targetCoords = combatTools.parseCoords(target);
                    var targetCont = server.getContinentFromCoords(targetCoords[0], targetCoords[1]);
                    var sourceCoords = combatTools.cityIdToCoords(city.getId());
                    var sourceCont = server.getContinentFromCoords(sourceCoords[0], sourceCoords[1]);

                    naval = (targetCont != sourceCont);
                }

                // Validate, whether is reachable by water
                if (naval) {
                    if (!city.getOnWater()) {
                        throw new Error("Unable to launch naval attack from land-locked castle");
                    }
                }

                // Get units for attack
                var attackUnits = null;
                if (attack == "fake") {
                    if (useScouts) {
                        // Try to use scouts
                        try {
                            attackUnits = combatTools.prepareScoutAttackUnits(availUnits, naval, true, false, minTS);
                            type = combatTools.SCOUT_ORDER_ID;
                        }
                        catch (ignored) {
                        }
                    }

                    // This is default or when scouting is not an option
                    if (attackUnits == null) {
                        attackUnits = combatTools.prepareFakeAttackUnits(availUnits, naval, minTS);
                    }
                } else if (attack == "scout") {
                    attackUnits = combatTools.prepareScoutAttackUnits(availUnits, naval, false, allowPartial, minTS);
                } else {
                    attackUnits = combatTools.prepareRealAttackUnits(availUnits, naval, attack == "demo", attack == "capture", useScouts, allowPartial, minTS);
                }

                // Determine attack time
                var attackType = combatTools.getMajorAttackType(attackUnits.units);
                var times = this.getAttackTimes();

                var attackTime = times[attackType];
                if (attackTime == null) {
                    throw new Error("Unknown time of the attack");
                }

                // Demo is always sieging
                if (attackType == "d") {
                    type = combatTools.SIEGE_ORDER_ID;
                }

                // Put it all together
                return {
                    target: target,
                    type: type,
                    units: attackUnits.units,
                    attackTS: attackUnits.totalTS,
                    timingType: combatTools.ARRIVAL_TIMING_ID,
                    time: attackTime,
                    isPartial: attackUnits.isPartial,
                    attackType: attackType
                };
            },
            copyTimes: function () {
                var value = this._magicTime.getValue();

                this._cavTime.setValue(value);
                this._infTime.setValue(value);
                this._siegeTime.setValue(value);
            },
            getMinAttackTS: function () {
                return 1;
            },
            resetMessage: function () {
                this._messageLabel.setValue("");
            },
            setMessage: function (text) {
                this._messageLabel.setValue(text || "");
            },
            getData: function () {
                var combatTools = paTweak.CombatTools;
                var data = {};

                // Get times
                data.times = {
                    magic: combatTools.convertGameTimeToUtc(this._magicTime.getValue()),
                    inf: combatTools.convertGameTimeToUtc(this._infTime.getValue()),
                    cav: combatTools.convertGameTimeToUtc(this._cavTime.getValue()),
                    siege: combatTools.convertGameTimeToUtc(this._siegeTime.getValue())
                };

                // Targets
                data.targets = [];

                this._rows.forEach(function (row) {
                    var value = row.getValue();
                    if (value != null) {
                        data.targets.push(value);
                    }
                });

                // Options
                data.includeActive = this._includeActive.getValue();
                data.allowPartial = this._allowPartial.getValue();
                data.useScouts = this._useScouts.getValue();
                data.useSmallestForFakes = this._useSmallestForFakes.getValue();
                data.excludeDefense = this._excludeDefenseCheck.getValue();
                data.forceMs = this._forceMsCheck.getValue();
                data.travelMode = this.getTravelMode();

                return data;
            },
            setData: function (data) {
                var _this = this;
                var combatTools = paTweak.CombatTools;

                // Reset
                this.reset();

                // Times
                if (data.times) {
                    var now = new Date().getTime();
                    if (data.times.magic && data.times.magic > now) this._magicTime.setValue(combatTools.convertUtcToGameTime(data.times.magic));
                    if (data.times.inf && data.times.inf > now) this._infTime.setValue(combatTools.convertUtcToGameTime(data.times.inf));
                    if (data.times.cav && data.times.cav > now) this._cavTime.setValue(combatTools.convertUtcToGameTime(data.times.cav));
                    if (data.times.siege && data.times.siege > now) this._siegeTime.setValue(combatTools.convertUtcToGameTime(data.times.siege));
                }

                // Targets
                if (data.targets && data.targets.length > 0) {
                    // Delete rows
                    this._rows.forEach(this._removeRow, this);
                    this._rows = [];

                    // Add new
                    data.targets.forEach(function (rowData) {
                        var row = _this.addRow();
                        row.setValue(rowData);
                    });
                }

                // Include active
                this._includeActive.setValue(data.includeActive != null ? data.includeActive : true);
                this._allowPartial.setValue(!!data.allowPartial);
                this._useScouts.setValue(data.useScouts != null ? data.useScouts : true);
                this._useSmallestForFakes.setValue(!!data.useSmallestForFakes);
                this._excludeDefenseCheck.setValue(!!data.excludeDefense);
                this._forceMsCheck.setValue(!!data.forceMs);
                this._travelModeGroup.setModelSelection([data.travelMode || "auto"]);
            },
            getStoragePath: function () {
                return "paTweak.ui.CombatWindow." + webfrontend.data.Player.getInstance().getId();
            },
            storeData: function () {
                try {
                    var path = this.getStoragePath();
                    var data = this.getData();
                    localStorage.setItem(path, JSON.stringify(data));
                    paDebug("CombatWindow data stored");
                }
                catch (e) {
                    paDebug("Unable to load CombatWindow data: " + e);
                }
            },
            loadData: function () {
                try {
                    var path = this.getStoragePath();
                    var data = JSON.parse(localStorage.getItem(path));

                    if (data != null) {
                        this.setData(data);
                        paDebug("CombatWindow data loaded");
                    }
                    else {
                        this.reset();
                        paDebug("CombatWindow data had no data to load");
                    }
                }
                catch (e) {
                    paDebug("Unable to load CombatWindow data: " + e);
                }
            }
        }
    });
});

