loader.addFinishHandler(function () {

    qx.Class.define("paTweak.ui.CombatWindowExport", {
        type: "singleton",
        extend: qx.ui.window.Window,
        construct: function () {
            this.base(arguments, "Commands Import/Export");
            this.buildUI();
        },
        statics: {
            ORDER_TYPES: {
                "1": "scout",
                "2": "plunder",
                "3": "assault",
                "4": "support",
                "5": "siege"
            },

            _formatTime: function (utcTime) {
                // Get time in server time
                var gameTime = paTweak.CombatTools.convertUtcToGameTime(utcTime, 1);
                var text = qx.lang.String.pad(String(gameTime.getUTCFullYear()), 4, "0") + "/";
                text += qx.lang.String.pad(String(gameTime.getUTCMonth() + 1), 2, "0") + "/";
                text += qx.lang.String.pad(String(gameTime.getUTCDate()), 2, "0") + " ";
                text += qx.lang.String.pad(String(gameTime.getUTCHours()), 2, "0") + ":";
                text += qx.lang.String.pad(String(gameTime.getUTCMinutes()), 2, "0") + ":";
                text += qx.lang.String.pad(String(gameTime.getUTCSeconds()), 2, "0");
                return text;
            },
            _parseTime: function (text) {
                var m = text.match(/^\s*(\d{4})\/?(\d{1,2})\/?(\d{1,2})\s+(\d{1,2}):(\d{1,2}):(\d{1,2})\s*$/);
                if (!m) {
                    return null;
                }

                var date = Date.UTC(m[1], m[2] - 1, m[3], m[4], m[5], m[6], 0);
                if (!isNaN(date)) {
                    // Note: Times are always in server time
                    return paTweak.CombatTools.convertGameTimeToUtc(new Date(date), 1);
                }
                else {
                    return null;
                }
            },
            dataToString: function (data, separator) {
                var segments = [];

                // Name
                var name = webfrontend.data.Server.getInstance().getName();
                segments.push(name.replace(/\s*\(.*\)\s*/, ""));

                // Times
                if (data.times) {
                    var now = new Date().getTime();
                    if (data.times.magic && data.times.magic > now) segments.push("Magic " + this._formatTime(data.times.magic));
                    if (data.times.cav && data.times.cav > now) segments.push("Cavalry " + this._formatTime(data.times.cav));
                    if (data.times.inf && data.times.inf > now) segments.push("Infantry " + this._formatTime(data.times.inf));
                    if (data.times.siege && data.times.siege > now) segments.push("Siege " + this._formatTime(data.times.siege));
                }

                // Targets
                if (data.targets && data.targets.length > 0) {
                    data.targets.forEach(function (target) {
                        var typeText = paTweak.ui.CombatWindowExport.ORDER_TYPES[target.type] || target.type;
                        var noteText = (target.note && target.note.length > 0) ? " - " + target.note : "";
                        segments.push(qx.lang.String.capitalize(target.target + " " + target.attack + " " + typeText) + noteText);
                    });
                }

                // Join
                return segments.join(separator);
            },
            parseData: function (text, separator) {
                var segments = text.split(separator);
                var data = {
                    times: {},
                    targets: []
                };

                // Go thru lines and parse them
                segments.forEach(function (segment) {
                    segment = paTweak.CombatTools.removeBBcode(segment).trim();
                    var time, m;

                    // Times
                    if (m = segment.match(/^magic\s+(.*)$/i)) {
                        time = paTweak.ui.CombatWindowExport._parseTime(m[1]);
                        if (time != null) {
                            data.times.magic = time;
                        }
                        return;
                    } else if (m = segment.match(/^infantry\s+(.*)$/i)) {
                        time = paTweak.ui.CombatWindowExport._parseTime(m[1]);
                        if (time != null) {
                            data.times.inf = time;
                        }
                        return;
                    } else if (m = segment.match(/^cavalry\s+(.*)$/i)) {
                        time = paTweak.ui.CombatWindowExport._parseTime(m[1]);
                        if (time != null) {
                            data.times.cav = time;
                        }
                        return;
                    } else if (m = segment.match(/^siege\s+(.*)$/i)) {
                        time = paTweak.ui.CombatWindowExport._parseTime(m[1]);
                        if (time != null) {
                            data.times.siege = time;
                        }
                        return;
                    }

                    // Target
                    var targetMatch = segment.match(/^(\d{1,3}:\d{1,3})\s+(fake|capture|demo|attack|scout)\s+(plunder|siege|assault|scout)\b\s*(.*)$/i);
                    if (targetMatch) {
                        var type = qx.lang.Object.getKeyFromValue(paTweak.ui.CombatWindowExport.ORDER_TYPES, targetMatch[3].toLowerCase());
                        data.targets.push({
                            target: targetMatch[1],
                            attack: targetMatch[2].toLowerCase(),
                            type: type,
                            note: (targetMatch[4] || "").replace(/^\s*-\s*/, "")
                        });
                    }
                });

                // Cleanup
                if (qx.lang.Object.getValues(data.times).length < 1) {
                    delete data.times;
                }
                if (data.targets.length < 1) {
                    delete data.targets;
                }

                return data;
            }
        },
        members: {
            _compactCheck: null,
            _contentText: null,
            _importButton: null,
            _exportButton: null,
            _closeButton: null,

            buildUI: function () {
                var app = qx.core.Init.getApplication();

                this.setLayout(new qx.ui.layout.VBox(5));
                this.set({allowMaximize: false, allowMinimize: false, showMaximize: false, showMinimize: false,
                    showStatusbar: false, showClose: false, contentPadding: 5, useMoveFrame: true, resizable: true});
                this.set({width: 250, height: 300});

                webfrontend.gui.Util.formatWinClose(this);

                // Note
                var note = new qx.ui.basic.Label("<em>Note: Time is always in Server Time</em>");
                note.setRich(true);
                this.add(note, {flex: 0});

                // Text area
                var contentText = this._contentText = new qx.ui.form.TextArea("");
                //this._contentText.set({});
                app.setElementModalInput(contentText);
                this.add(contentText, {flex: 1});

                // Compact
                var compactCheck = this._compactCheck = new qx.ui.form.CheckBox("Compact");
                compactCheck.set({toolTipText: "Use single-line format."});
                compactCheck.addListener("changeValue", this.exportData, this);
                this.add(compactCheck, {flex: 0});

                // Buttons
                var exportButton = this._exportButton = new qx.ui.form.Button("Refresh");
                exportButton.set({width: 80, toolTipText: "Generate text from the Advanced Commands window."});
                exportButton.addListener("execute", this.exportData, this);

                var importButton = this._importButton = new qx.ui.form.Button("Import!");
                importButton.set({width: 80, toolTipText: "Import data into the dialog."});
                importButton.addListener("execute", this.importData, this);

                var closeButton = this._closeButton = new qx.ui.form.Button("Close");
                closeButton.set({width: 80, toolTipText: "Closes the dialog."});
                closeButton.addListener("execute", this.hide, this);

                var buttonsRow = new qx.ui.container.Composite();
                buttonsRow.setLayout(new qx.ui.layout.HBox(5));
                buttonsRow.set({alignX: "right"});

                buttonsRow.add(exportButton);
                buttonsRow.add(importButton);
                buttonsRow.add(closeButton);
                this.add(buttonsRow, {flex: 0});
            },
            exportData: function () {
                this._contentText.setValue("");

                var sep = this._compactCheck.getValue() ? "|" : "\n";
                var data = paTweak.ui.CombatWindow.getInstance().getData();
                var text = paTweak.ui.CombatWindowExport.dataToString(data, sep);

                this._contentText.setValue(text);
                this._contentText.selectAllText();
                this._contentText.focus();
            },
            importData: function () {
                var text = this._contentText.getValue();
                var data = paTweak.ui.CombatWindowExport.parseData(text, /[\n|]/);

                paTweak.ui.CombatWindow.getInstance().setData(data);
            }
        }
    });
});
