loader.addFinishHandler(function () {

    qx.Class.define("paTweak.ui.IdleRaidUnitsTable", {
        extend: qx.ui.table.Table,
        implement: [webfrontend.net.IUpdateConsumer],
        construct: function () {
            var tableModel = new qx.ui.table.model.Simple();
            var columnNames = ["CityID", "TS", "Type", "City", "Ref", "Cont", "Coords"];

            tableModel.setColumns(columnNames);

            var custom =
            {
                tableColumnModel: function (obj) {
                    return new qx.ui.table.columnmodel.Resize(obj);
                }
            };

            qx.ui.table.Table.call(this, tableModel, custom);

            this.addListener("cellClick", this.onCellClick, this);

            var columnModel = this.getTableColumnModel();
            columnModel.setColumnVisible(0, false);
            var linkStyle = new qx.ui.table.cellrenderer.Default();
            linkStyle.setDefaultCellStyle("text-decoration:underline;color:blue");
            columnModel.setDataCellRenderer(3, linkStyle);
            columnModel.setDataCellRenderer(6, linkStyle);
            //this.refresh();
        },
        members: {
            onCellClick: function (event) {

                var cityID;
                var spl;
                var x;
                var y;

                switch (event.getColumn()) {
                    case 3:
                    {
                        cityID = this.getTableModel().getValue(0, event.getRow());
                        spl = this.getTableModel().getValue(6, event.getRow()).split(":");
                        x = Number(spl[0]);
                        y = Number(spl[1]);
                        webfrontend.data.City.getInstance().setRequestId(cityID);
                        webfrontend.gui.Util.showMapModeViewPos('r', 0, x, y);
                    }
                        break;
                    case 6:
                    {
                        cityID = this.getTableModel().getValue(0, event.getRow());
                        spl = this.getTableModel().getValue(6, event.getRow()).split(":");
                        x = Number(spl[0]);
                        y = Number(spl[1]);
                        webfrontend.gui.Util.showMapModeViewPos('r', 0, x, y);
                    }
                        break;
                }
            },
            updateCityTS: function (cid, count) {
                var tm = this.getTableModel();
                for (var i = 0; i < tm.getRowCount(); i++) {
                    if (tm.getValue(0, i) == cid) {
                        tm.setValue(1, i, count);
                        break;
                    }
                }
            },
            removeDefoConsumer: function () {
                removeConsumer("DEFO", this.dispatchResults);
            },
            refresh: function () {
                var tm = this.getTableModel();
                tm.removeRows(0, tm.getRowCount());
                tm.addRows([
                    [0, "Loading..."]
                ]);
                var rw = paTweak.ui.RaidingWindow.getInstance();
                this.removeDefoConsumer();
                addConsumer("DEFO", this.dispatchResults, this);
            },
            getRequestDetails: function (details) {
                return "a";
            },
            dispatchResults: function (results, thisObj) {
                if (results == null)
                    return;
                var resMain = webfrontend.res.Main.getInstance();
                var sI = webfrontend.data.Server.getInstance();
                var pC = webfrontend.data.Player.getInstance().cities;
                var tm = thisObj.getTableModel();
                var CI = webfrontend.data.City.getInstance();
                var curCityId = CI.getId();
                var rw = paTweak.ui.RaidingWindow.getInstance();
                var bossT = rw.bossRaider.t;
                var bossTcount = 0;
                var idleCities = [];
                tm.removeRows(0, tm.getRowCount());
                var excludeIfTxt = rw.excludeIf.getValue();
                var excludeShips = rw.excludeShips.getValue();
                var excludeRefs = "";
                var hasExcludes = false;
                var excludeTs = Number(rw.excludeTs.getValue());
                if (excludeIfTxt.length > 0) {
                    excludeRefs = excludeIfTxt.split(',');
                    for (var ii = 0; ii < excludeRefs.length; ++ii) {
                        if (excludeRefs[ii].length > 0) {
                            hasExcludes = true;
                            break;
                        }
                    }
                }
                for (var i = 0; i < results.length; i++) {
                    var result = results[i];
                    if (result.hasOwnProperty("c")) {
                        for (var j = 0; j < result.c.length; j++) {
                            var command = result.c[j];
                            if (command.i == 0) {
                                var x = result.i & 0xffff;
                                var y = result.i >> 16;
                                var first = true;
                                var unitStr = "";
                                var ts = 0;
                                for (var k = 0; k < command.u.length; k++) {
                                    var unitInfo = resMain.units[command.u[k].t];
                                    var count = command.u[k].c;
                                    if (count > 0 && unitInfo.c > 0 && (!excludeShips || unitInfo.ls) && rw._st) {
                                        if (!first) {
                                            unitStr += ",";
                                        }
                                        unitStr += unitShortName(command.u[k].t);
                                        ts += count * unitInfo.uc;
                                        first = false;
                                        if (result.i == curCityId) {
                                            bossTcount += count;
                                        }
                                    }
                                }
                                if (!first) {
                                    //var columnNames = ["CityID", "TS", "Type", "City", "Ref", "Cont", "Coords"];
                                    var cont = sI.getContinentFromCoords(x, y);
                                    var ref = "";
                                    if (pC.hasOwnProperty(result.i))
                                        ref = pC[result.i].reference;
                                    var exclude = (excludeTs > 0 && ts < excludeTs);
                                    if (hasExcludes) {
                                        for (var ii = 0; !exclude && ii < excludeRefs.length; ++ii) {
                                            exclude = ref.indexOf(excludeRefs[ii]) >= 0;
                                        }
                                    }
                                    if (!exclude) {
                                        idleCities.push([result.i, ts, unitStr, result.n, ref, cont, x.toString() + ":" + y.toString()]);
                                    }
                                }
                            }
                        }
                    }
                    if (result.i == curCityId) {
                        rw.bossUnitLabel.setValue(rw.formatNumber(bossTcount));
                    }
                }
                tm.setData(idleCities);
                tm.sortByColumn(1, false);
                if (!rw.autoUpdate.getValue()) {
                    thisObj.removeDefoConsumer();
                }
                rw.nextIdleCityButton.setEnabled(true);
            }
        }
    });
});