loader.addFinishHandler(function () {

    function refreshItems() {
        var dialog = paTweak.ui.PalaceItemsWindow.getInstance();
        dialog.fillItemRow();
    }

    function waitForItem() {
        window.setTimeout(refreshItems, 3000);
    }

    qx.Class.define("paTweak.ui.PalaceItemsWindow", {
        type: "singleton",
        extend: qx.ui.window.Window,
        construct: function () {
            this.base(arguments, 'Use Palace Items');
            this.buildUI();

            // Refresh info every time
            this.addListener("appear", this.fillItemRow, this);
        },
        members: {
            _returnTime: null,
            palaceItemMessageLabel: null,
            palaceItemRow: null,
            buildUI: function () {
                var app = qx.core.Init.getApplication();

                this.setLayout(new qx.ui.layout.VBox(2));
                this.set({allowMaximize: false, allowMinimize: false, showMaximize: false, showMinimize: false,
                    showStatusbar: false, showClose: false, contentPadding: 5, useMoveFrame: true, resizable: false});
                this.setWidth(350);
                webfrontend.gui.Util.formatWinClose(this);

                var wcLabel = new qx.ui.basic.Label("Select the city then click the item to apply");
                wcLabel.set({font: "bold"});
                this.add(wcLabel);

                this.palaceItemRow = new qx.ui.container.Composite();
                this.palaceItemRow.setLayout(new qx.ui.layout.HBox());
                this.add(this.palaceItemRow);

                var row = new qx.ui.container.Composite();
                row.setLayout(new qx.ui.layout.HBox());

                this.palaceItemMessageLabel = new qx.ui.basic.Label("");
                this.palaceItemMessageLabel.set({minWidth: 30, allowGrowX: true, font: "bold", textColor: "red", toolTipText: " "});
                row.add(this.palaceItemMessageLabel);

                this.add(row);

                var row = new qx.ui.container.Composite();
                row.setLayout(new qx.ui.layout.HBox());

                // Close button
                var closeButton = new qx.ui.form.Button("Close");
                closeButton.addListener("execute", this.hide, this);

                row.add(closeButton);

                this.add(row);
            },
            fillItemRow: function () {
                this.palaceItemRow.removeAll();
                this.palaceItemMessageLabel.setValue("");
                var i = webfrontend.data.Inventory.getInstance();
                var bS = webfrontend.res.Main.getInstance();
                var itemCount = 0;
                var title = webfrontend.data.Player.getInstance().getTitle();
                for (var ii = 178; ii < 188; ++ii) {
                    var itemUseable = false;
                    switch (ii) {
                        case 178:
                        case 183:
                            itemUseable = title >= 10;
                            break;
                        case 179:
                        case 184:
                            itemUseable = title >= 9;
                            break;
                        case 180:
                        case 185:
                            itemUseable = title >= 8;
                            break;
                        case 181:
                        case 186:
                            itemUseable = title >= 7;
                            break;
                        case 182:
                        case 187:
                            itemUseable = title >= 6;
                            break;
                    }
                    if (i.hasItem(ii) && itemUseable) {
                        ++itemCount;
                        var inv = i.getInventorySorted();
                        var num = 0;
                        for (var ix = 0; ix < inv.length; ++ix) {
                            if (inv[ix].id == ii) {
                                num = inv[ix].count;
                                break;
                            }
                        }
                        var itemImg = new qx.ui.basic.Image("webfrontend/" + bS.imageFiles[bS.items[ii].i]);
                        itemImg.itemId = String(ii);
                        itemImg.thisObj = this;
                        itemImg.set({padding: 5, toolTipText: "You own " + String(num) + " artifact" + (num > 1 ? "s" : "") + " <br/>" + bS.items[ii].dn + "<br/>" + bS.items[ii].sds });
                        itemImg.setWidth(40);
                        itemImg.setHeight(40);
                        itemImg.setScale(true);
                        this.palaceItemRow.add(itemImg);
                        itemImg.addListener("click", this.useItem, this);
                    }
                }
                if (itemCount == 0) {
                    this.palaceItemMessageLabel.setValue("No useable palace items found");
                }
                /*
                 178 - Valorite Arch
                 179 - Verite Arch
                 180 - Platinum Arch
                 181 - Golden Arch
                 182 - Silver Arch

                 183 - Valorite Pillar
                 184 - Verite Pillar
                 185 - Platinum Pillar
                 186 - Golden Pillar
                 187 - Silver Pillar
                 */
            },
            useItem: function (e) {
                var currentTarget = e.getCurrentTarget();
                var itemId = currentTarget.itemId;
                var app = qx.core.Init.getApplication();
                var selectedCity = (app.cityDetailView || app.getCityDetailView()).city;
                var myAllianceName = webfrontend.data.Alliance.getInstance().getName().toLowerCase();
                if (!selectedCity || !selectedCity.get_IsEnlighted()) {
                    this.palaceItemMessageLabel.setValue("Select an enlightened city");
                    return;
                }
                if (selectedCity.get_AllianceName().iCompare(myAllianceName)) {
                    this.palaceItemMessageLabel.setValue("Enlightend city must belong to " + myAllianceName);
                    return;
                }
                var i = webfrontend.data.Inventory.getInstance();
                if (!i.hasItem(itemId)) {
                    this.palaceItemMessageLabel.setValue("You do not own the item selected");
                    return;
                }

                var bS = webfrontend.res.Main.getInstance();
                var coords = selectedCity.get_Coordinates();
                this.palaceItemMessageLabel.setValue("Please wait, using 1 " + bS.items[itemId].dn);
                var commandManager = webfrontend.net.CommandManager.getInstance();
                commandManager.sendCommand("UseItem", {"itemid": itemId, "amount": 1, "target": [
                    {"t": bS.items[itemId].tt, "i": coords}
                ]}, currentTarget.thisObj, waitForItem);
            }
        }
    });
});