loader.addFinishHandler(function () {

    qx.Class.define("paTweak.ui.components.AttackOrder", {
        extend: qx.ui.container.Composite,
        construct: function () {
            this.base(arguments);

            var combatTools = paTweak.CombatTools;
            var PLUNDER = {label: "Plunder", type: combatTools.PLUNDER_ORDER_ID};
            var SIEGE = {label: "Siege", type: combatTools.SIEGE_ORDER_ID};
            var ASSAULT = {label: "Assault", type: combatTools.ATTACK_ORDER_ID};
            var SCOUT = {label: "Scout", type: combatTools.SCOUT_ORDER_ID};

            this.ATTACK_ACTIONS = [];
            this.ATTACK_ACTIONS.push({name: "fake", label: "Fake", allowed: [SIEGE, PLUNDER, SCOUT], tooltip: "Minimal troop count will be sent."});
            this.ATTACK_ACTIONS.push({name: "capture", label: "Capture", allowed: [SIEGE, ASSAULT],
                tooltip: "Barons will be included in the attack, if available. No Catapults or Galleons will be sent, only Rams."});
            this.ATTACK_ACTIONS.push({name: "demo", label: "Demolish", allowed: [SIEGE, ASSAULT], tooltip: "Catapults and Galleons will be included in the attack."});
            this.ATTACK_ACTIONS.push({name: "attack", label: "Attack", allowed: [SIEGE, PLUNDER, ASSAULT],
                tooltip: "Simple attack, no Catapults, Galleons or Barons will be included. Rams will be used, if available."});
            this.ATTACK_ACTIONS.push({name: "scout", label: "Scout", allowed: [SCOUT], tooltip: "Only scouts will be sent."});

            this.buildUI();
            this.selectAction(this.ATTACK_ACTIONS[0]);
        },
        events: {
            attack: "qx.event.type.Data",
            changeValue: "qx.event.type.Event"
        },
        members: {
            ATTACK_ACTIONS: null,

            _attackButton: null,
            _actionButton: null,
            _coordsText: null,
            _toggleButton: null,
            _noteText: null,
            _counterLabel: null,

            _selectedAction: null,
            _selectedTypeIndex: -1,
            _applyingValue: false,

            buildUI: function () {
                var _this = this;
                var app = qx.core.Init.getApplication();
                this.setLayout(new qx.ui.layout.HBox(5));

                // Attack button
                var actionMenu = new qx.ui.menu.Menu();
                this.ATTACK_ACTIONS.forEach(function (action) {
                    var menuButton = new qx.ui.menu.Button(action.label);
                    menuButton.addListener("execute", function () {
                        _this.selectAction(action);
                    });
                    actionMenu.add(menuButton);
                });

                this._attackButton = new qx.ui.form.Button("[Select]");
                this._attackButton.set({appearance: "button-text-small", width: 80});
                this._attackButton.addListener("execute", this.fireAttack, this);

                this._actionButton = new qx.ui.form.MenuButton("▼", null, actionMenu);
                this._actionButton.set({appearance: "button-text-small", width: 20});

                var attackControl = new qx.ui.container.Composite();
                attackControl.setLayout(new qx.ui.layout.HBox(1));
                attackControl.add(this._attackButton);
                attackControl.add(this._actionButton);

                // Toggle button
                this._toggleButton = new qx.ui.form.Button("[Select]");
                this._toggleButton.set({appearance: "button-text-small", width: 60, toolTipText: "Siege Engines and Baron will always siege the target, regardless the option."});
                this._toggleButton.addListener("execute", this.onModeToggle, this);

                // Coords
                this._coordsText = new qx.ui.form.TextField();
                this._coordsText.set({width: 80, marginTop: 1, maxLength: 40, toolTipText: "Coordinates in xxx:yyy format."});
                app.setElementModalInput(this._coordsText);

                this._coordsText.addListener("changeValue", this.onNormalizeCoords, this);
                this._coordsText.addListener("changeValue", this.fireChangeValue, this);

                // Note
                this._noteText = new qx.ui.form.TextField();
                this._noteText.set({width: 220, toolTipText: "Just a note."});
                this._noteText.addListener("changeValue", this.fireChangeValue, this);
                app.setElementModalInput(this._noteText);

                this._counterLabel = new qx.ui.basic.Label();
                this._counterLabel.set({minWidth: 30, allowGrowX: true, toolTipText: "Indicative count of attacks you have sent to this target. DblClick to remove last entry. I=Infrantry, C=Cavalry, M=Magic, D=Siege Engines"});
                this._counterLabel.addListener("dblclick", this.removeLastCount, this);

                // Add to page
                this.add(attackControl);
                this.add(this._coordsText);
                this.add(this._toggleButton);
                this.add(this._noteText);
                this.add(this._counterLabel);
            },
            selectAction: function (action) {
                this._selectedAction = action;
                this._attackButton.setLabel(action.label.toUpperCase());
                this._attackButton.setToolTipText(action.tooltip);

                // Update mode
                this._selectedTypeIndex = -1;
                this.onModeToggle();

                // Note: Change event is fired in onModeToggle
            },
            onModeToggle: function () {
                var allowed = this._selectedAction.allowed;

                this._selectedTypeIndex++;
                if (this._selectedTypeIndex >= allowed.length) {
                    this._selectedTypeIndex = 0;
                }

                // Set label
                this._toggleButton.setLabel(allowed[this._selectedTypeIndex].label);

                // Fire change event
                this.fireChangeValue();
            },
            onNormalizeCoords: function (e) {
                var str = paTweak.CombatTools.normalizeCoords(e.getData());

                if (str != null && str != e.getData()) {
                    e.stopPropagation();
                    this._coordsText.setValue(str);
                }
            },
            fireAttack: function () {
                var value = this.getValue();

                if (value != null) {
                    // Fire the event
                    this.fireDataEvent("attack", value);
                }
            },
            fireChangeValue: function () {
                if (!this._applyingValue) {
                    this.fireEvent("changeValue");
                }
            },
            setAttackEnabled: function (value) {
                attackButton.setEnabled(value);
            },
            getValue: function () {
                // Get target
                var coords = paTweak.CombatTools.normalizeCoords(this._coordsText.getValue());
                var type = this._selectedAction.allowed[this._selectedTypeIndex];
                var note = (this._noteText.getValue() || "").trim();

                if (coords == null || type == null) {
                    return null;
                }

                // Return attack detail
                return {
                    attack: this._selectedAction.name,
                    type: type.type,
                    target: coords,
                    note: note
                };
            },
            setValue: function (data) {
                if (data == null) {
                    // Defaults
                    data = {fake: true};
                }

                try {
                    this._applyingValue = true;

                    // Action
                    var action = this._actionByName(data.attack);
                    this.selectAction(action);

                    // Type (TODO do it better)
                    var allowed = this._selectedAction.allowed;
                    this._selectedTypeIndex = 0;
                    for (var i = 0; i < allowed.length; i++) {
                        if (allowed[i].type == data.type) {
                            this._selectedTypeIndex = i;
                            break;
                        }
                    }

                    this._toggleButton.setLabel(allowed[this._selectedTypeIndex].label);

                    // Coords
                    var coords = paTweak.CombatTools.normalizeCoords(data.target);
                    this._coordsText.setValue(coords);

                    // Note
                    this._noteText.setValue(data.note || "");
                } finally {
                    this._applyingValue = false;
                }

                // Fire change event
                this.fireChangeValue();
            },
            setActionEnabled: function (value) {
                this._attackButton.setEnabled(value);
            },
            getActionEnabled: function () {
                return this._attackButton.getEnabled();
            },
            addCount: function (type) {
                var old = this._counterLabel.getValue() || "";
                this._counterLabel.setValue(old + type);
            },
            removeLastCount: function () {
                var old = this._counterLabel.getValue() || "";
                if (old.length > 0) {
                    this._counterLabel.setValue(old.substr(0, old.length - 1));
                }
            },
            resetCount: function () {
                this._counterLabel.resetValue();
            },
            _actionByName: function (name) {
                for (var i = 0; i < this.ATTACK_ACTIONS.length; i++) {
                    if (this.ATTACK_ACTIONS[i].name == name) {
                        return this.ATTACK_ACTIONS[i];
                    }
                }

                // Return fake as default
                return this.ATTACK_ACTIONS[0];
            }
        }
    });
});
