loader.addFinishHandler(function () {

    qx.Class.define("paTweak.ui.components.TimePicker", {
        extend: qx.ui.container.Composite,
        construct: function (caption) {
            this.base(arguments);
            this.buildUI(caption);
        },
        properties: {
            value: { check: "Date", init: new Date(0), apply: "_applyValue" }
        },
        events: {
            changeValue: "qx.event.type.Data"
        },
        members: {
            _dateSelect: null,
            _hourText: null,
            _minuteText: null,
            _secondText: null,

            _applyingValue: false,
            _updatingValue: false,

            buildUI: function (caption) {
                var app = qx.core.Init.getApplication();
                this.setLayout(new qx.ui.layout.HBox(5));

                // Caption
                if (caption != null) {
                    var captionLabel = new qx.ui.basic.Label(caption);
                    captionLabel.set({width: 60, allowGrowX: false});
                    this.add(captionLabel);
                }

                this._hourText = new qx.ui.form.TextField("0");
                this._hourText.set({width: 26, maxLength: 2});
                this._hourText.addListener("changeValue", this._onValidateHour, this._hourText);
                app.setElementModalInput(this._hourText);
                this.add(this._hourText);

                this._minuteText = new qx.ui.form.TextField("0");
                this._minuteText.set({width: 26, maxLength: 2});
                this._minuteText.addListener("changeValue", this._onValidateMinute, this._minuteText);
                app.setElementModalInput(this._minuteText);
                this.add(this._minuteText);

                this._secondText = new qx.ui.form.TextField("0");
                this._secondText.set({width: 26, maxLength: 2});
                this._secondText.addListener("changeValue", this._onValidateMinute, this._secondText);
                app.setElementModalInput(this._secondText);
                this.add(this._secondText);

                this._dateSelect = new qx.ui.form.SelectBox();
                this._dateSelect.set({width: 90});
                this._dateSelect.add(new qx.ui.form.ListItem("Today", null, 0));
                this._dateSelect.add(new qx.ui.form.ListItem("Tomorrow", null, 1));
                this._dateSelect.add(new qx.ui.form.ListItem("2 Days", null, 2));
                this._dateSelect.add(new qx.ui.form.ListItem("3 Days", null, 3));
                this._dateSelect.add(new qx.ui.form.ListItem("4 Days", null, 4));
                this._dateSelect.add(new qx.ui.form.ListItem("5 Days", null, 5));
                this._dateSelect.add(new qx.ui.form.ListItem("6 Days", null, 6));
                this._dateSelect.add(new qx.ui.form.ListItem("7 Days", null, 7));
                this.add(this._dateSelect);

                // changeValue listeners
                this._hourText.addListener("changeValue", this._updateValue, this);
                this._minuteText.addListener("changeValue", this._updateValue, this);
                this._secondText.addListener("changeValue", this._updateValue, this);
                this._dateSelect.addListener("changeSelection", this._updateValue, this);
            },
            fireChangeValue: function () {
                this.fireDataEvent("changeValue", this.getValue());
            },
            _applyValue: function (value) {
                if (this._updatingValue) {
                    return;
                }

                // We need to get date difference
                var gameNow = webfrontend.Util.getCurrentTime().getTime();
                var totalDaysNow = Math.floor(gameNow / (24 * 3600 * 1000));
                var totalDaysValue = Math.floor(value.getTime() / (24 * 3600 * 1000));
                var daysOffset = totalDaysValue - totalDaysNow;

                // Update UI
                try {
                    this._applyingValue = true;
                    this._hourText.setValue(String(value.getUTCHours()));
                    this._minuteText.setValue(String(value.getUTCMinutes()));
                    this._secondText.setValue(String(value.getUTCSeconds()));
                    this._dateSelect.setModelSelection([daysOffset]);
                }
                finally {
                    this._applyingValue = false;
                }

                this.fireChangeValue();
            },
            _updateValue: function () {
                if (this._applyingValue) {
                    return;
                }

                // Parse fields
                var hours = Number(this._hourText.getValue());
                var minutes = Number(this._minuteText.getValue());
                var seconds = Number(this._secondText.getValue());
                var dayValue = this._dateSelect.getSelection()[0].getModel();
                var dateOffset = Number(dayValue);

                // This function is a bit wierd, returned instance UTC value
                // corresponds to visible time to user.
                var gameNow = webfrontend.Util.getCurrentTime().getTime();
                gameNow += dateOffset * 24 * 3600 * 1000;

                // Prepare return date object
                var date = new Date(gameNow);
                date.setUTCHours(hours);
                date.setUTCMinutes(minutes);
                date.setUTCSeconds(seconds);
                date.setUTCMilliseconds(0);

                try {
                    this._updatingValue = true;
                    this.setValue(date);
                }
                finally {
                    this._updatingValue = false;
                }

                this.fireChangeValue();
            },
            _onValidateHour: function (e) {
                var num = Math.floor(Number(e.getData()));
                if (num > 23) {
                    e.stopPropagation();
                    this.setValue("23");
                }
                else if (num < 0 || isNaN(num)) {
                    e.stopPropagation();
                    this.setValue("0");
                }
                else if (String(num) != e.getData()) {
                    e.stopPropagation();
                    this.setValue(String(num));
                }
            },
            _onValidateMinute: function (e) {
                var num = Math.floor(Number(e.getData()));
                if (num > 59) {
                    e.stopPropagation();
                    this.setValue("59");
                }
                else if (num < 0 || isNaN(num)) {
                    e.stopPropagation();
                    this.setValue("0");
                }
                else if (String(num) != e.getData()) {
                    e.stopPropagation();
                    this.setValue(String(num));
                }
            }
        }
    });
});
